package edu.cs2335.SWATLP.client.clientdata;

import edu.cs2335.SWATLP.client.spacegame.PlayerShip;

/**
 * LunarLander data stuff.
 */
public class LunarLander extends Thread implements Runnable {

    /** */
    private boolean running;

    /** The acceleration rate of the boost */
    private static final double BOOSTSPEED = 5;

    /** the side movement speed */
    private static final double SIDESPEED = 5.0;

    /** Is the key for boosting upwards being pressed? */
    private int boostkey;

    /** the current time for movement */
    private long currTime;

    /** The land. */
    private int[] land;

    /** The landing spot - this ranges from this value to this value + 49 */
    private int landingspot;

    /** the time since last movement */
    private long lastTime;

    /** landed */
    private boolean landed;

    /** height of the landing zone */
    private int landingheight;

    /** crashed */
    private boolean crashed;

    /**
     * Is the key for boosting upwards being pressed? -1 = left 0 = none 1 =
     * right
     */
    private int sidekey;

    /** The player's ship. */
    private PlayerShip theship;

    /** current speed */
    private double speed;

    /** */
    private ApproachData data;

    /**
     * Comment for <code>collision</code>
     */
    private boolean collision;

    /**
     * LunarLander Constructor
     * 
     * @param thedata
     *            data
     * @param theship
     *            The ship in the lunar lander.
     */
    public LunarLander(ApproachData thedata, PlayerShip theship) {
        data = thedata;
        this.theship = theship;
        theship.setPosition(49, 49);
        land = new int[800]; // length of the land
        lastTime = System.currentTimeMillis();
        currTime = System.currentTimeMillis();
        boostkey = 0;
        sidekey = 0;
        crashed = false;
        landed = false;
        speed = 0;
        running = false;
        collision = false;
    }

    /**
     * Generates the land.
     */
    public void generateLand() {
        land[0] = (int) (Math.random() * 50) + 625;
        for (int x = 1; x < 800; x++) {
            land[x] = land[x - 1] + (int) ((Math.random() * 6) - 3);
            if (land[x] > 590) {
                land[x] -= 2;
            }
        }
        landingspot = (int) ((Math.random() * 700) + 25);
        for (int x = landingspot; x < landingspot + 50; x++) {
            land[x] = 750;
        }
        landingheight = 750;
    }

    /**
     * Gets called frequently to determine the ship's new position.
     */
    public void movement() {
        // TODO Tons of shit!

        if (this.theship.isFiring()) {
            this.boostkey = 1;
            //System.out.println("BOOST SET");
        } else {
            this.boostkey = 0;
            //System.out.println("BOOST OFF");
        }

        if (this.theship.isTurningleft()) {
            this.sidekey = -1;
        } else if (this.theship.isTurningright()) {
            this.sidekey = 1;
        } else {
            sidekey = 0;
        }

        // gravity = 1.5
        /*
         * if (data.getFuelCurrent() <= 0) { sidekey = 0; boostkey = 0; }
         */

        double t = (currTime - lastTime) / 100.0;
        double ay = 1.5 - BOOSTSPEED * boostkey;
        //System.out.println("AY: " + ay);
        double vx = 0 + (SIDESPEED * sidekey);

        int y = (int) Math.floor(theship.getYLoc() + speed * t + .5 * ay * t
                * t);

        speed = speed + ay * t;
        //System.out.println("SPEED: " + speed);
        //System.out.println("Y: " + theship.getYLoc());
        //System.out.println("Real Y: " + realY());

        int x = (int) Math.floor(theship.getXLoc() + vx * t);

        if (theship.getYLoc() >= land[(int) theship.getXLoc()] 
                                      && boostkey == 0) {
            speed = 0;
            y = (int) theship.getYLoc();
            x = (int) theship.getXLoc();
        }

        if (collision) {
            speed = 0;
            x = (int) theship.getXLoc();
            y = land[(int) theship.getXLoc()] - theship.getScreenHeight();
        }

        if (x < 0) {
            x = 0;
        }

        if (y < 0) {
            y = 0;
            speed = 0;
        }

        theship.setPosition((double) x, (double) y);
    }

    /**
     *  
     */
    public void run() {
        running = true;
        generateLand();
        this.theship.setPosition(300, 49);
        //System.out.println("HP: " + this.data.getHitpointsCurrent());
        do {
            try {
                sleep(10);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            currTime = System.currentTimeMillis();
            movement();
            landed = checkLanded();
            checkCollision();
            crashed = checkCrashed();
            lastTime = currTime;
        } while (!landed && !crashed);
       /* if (landed) {
            //System.out.println("LANDED");
        }
        if (crashed) {
            //System.out.println("CRASHED");
        }*/
    }

    /**
     *  
     */
    private void checkCollision() {
        if (!landed) {
            if ((int) theship.getYLoc() > land[(int) theship.getXLoc()]) {
                collision = true;
                if (speed > 30) {
                    //TODO theship.takeDamage((int) speed);
                    System.out.println("TAKE DAMAGE");
                }
                speed = 0;
            }
        }
    }

    /**
     * @return crashed
     */
    private boolean checkCrashed() {
        if (!landed) {
            if (data.getHitpointsCurrent() <= 0) {
                return theship.getYLoc() > this.land[(int) theship.getXLoc()];
            }
        }
        return false;
    }

    /**
     * @return landed
     */
    private boolean checkLanded() {
        if (theship.getXLoc() > this.landingspot
                && theship.getXLoc() < this.landingspot + 49) {
            //System.out.println("check landing zone");
            if (data.getHitpointsCurrent() > 0) {
                return (theship.getYLoc() > landingheight);
            }
        }
        return false;
    }

    /**
     * @return running
     */
    public boolean isRunning() {
        return running;
    }

    /**
     * @return done
     */
    public boolean isDone() {
        return (crashed || landed);
    }

    /**
     * @param x
     *            x
     * @return land[x]
     */
    public int getLand(int x) {
        return land[x];
    }

    /**
     * @return start of landing
     */
    public int getSafe() {
        return landingspot;
    }

    /**
     * @return Returns the bOOSTSPEED.
     */
    public static double getBOOSTSPEED() {
        return BOOSTSPEED;
    }
    /**
     * @return Returns the sIDESPEED.
     */
    public static double getSIDESPEED() {
        return SIDESPEED;
    }
    /**
     * @return Returns the boostkey.
     */
    public int getBoostkey() {
        return boostkey;
    }
    /**
     * @param boostkey The boostkey to set.
     */
    public void setBoostkey(int boostkey) {
        this.boostkey = boostkey;
    }
    /**
     * @return Returns the collision.
     */
    public boolean isCollision() {
        return collision;
    }
    /**
     * @param collision The collision to set.
     */
    public void setCollision(boolean collision) {
        this.collision = collision;
    }
    /**
     * @return Returns the crashed.
     */
    public boolean isCrashed() {
        return crashed;
    }
    /**
     * @param crashed The crashed to set.
     */
    public void setCrashed(boolean crashed) {
        this.crashed = crashed;
    }
    /**
     * @return Returns the currTime.
     */
    public long getCurrTime() {
        return currTime;
    }
    /**
     * @param currTime The currTime to set.
     */
    public void setCurrTime(long currTime) {
        this.currTime = currTime;
    }
    /**
     * @return Returns the data.
     */
    public ApproachData getData() {
        return data;
    }
    /**
     * @param data The data to set.
     */
    public void setData(ApproachData data) {
        this.data = data;
    }
    /**
     * @return Returns the land.
     */
    public int[] getLand() {
        return land;
    }
    /**
     * @param land The land to set.
     */
    public void setLand(int[] land) {
        this.land = land;
    }
    /**
     * @return Returns the landed.
     */
    public boolean isLanded() {
        return landed;
    }
    /**
     * @param landed The landed to set.
     */
    public void setLanded(boolean landed) {
        this.landed = landed;
    }
    /**
     * @return Returns the landingheight.
     */
    public int getLandingheight() {
        return landingheight;
    }
    /**
     * @param landingheight The landingheight to set.
     */
    public void setLandingheight(int landingheight) {
        this.landingheight = landingheight;
    }
    /**
     * @return Returns the landingspot.
     */
    public int getLandingspot() {
        return landingspot;
    }
    /**
     * @param landingspot The landingspot to set.
     */
    public void setLandingspot(int landingspot) {
        this.landingspot = landingspot;
    }
    /**
     * @return Returns the lastTime.
     */
    public long getLastTime() {
        return lastTime;
    }
    /**
     * @param lastTime The lastTime to set.
     */
    public void setLastTime(long lastTime) {
        this.lastTime = lastTime;
    }
    /**
     * @return Returns the sidekey.
     */
    public int getSidekey() {
        return sidekey;
    }
    /**
     * @param sidekey The sidekey to set.
     */
    public void setSidekey(int sidekey) {
        this.sidekey = sidekey;
    }
    /**
     * @return Returns the speed.
     */
    public double getSpeed() {
        return speed;
    }
    /**
     * @param speed The speed to set.
     */
    public void setSpeed(double speed) {
        this.speed = speed;
    }
    /**
     * @return Returns the theship.
     */
    public PlayerShip getTheship() {
        return theship;
    }
    /**
     * @param theship The theship to set.
     */
    public void setTheship(PlayerShip theship) {
        this.theship = theship;
    }
    /**
     * @param running The running to set.
     */
    public void setRunning(boolean running) {
        this.running = running;
    }
}