package edu.cs2335.SWATLP.client.gui;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

/**
 * @author thaile
 */
public class ChatUtil {
    /**
     * This is the max number of messages to hold
     */
    public static final int MAX_SIZE = 10;

    /**
     * List to hold the chatmessages
     */
    private ArrayList chatBuffer;

    /**
     * The number of msg in the buffer
     */
    private int msgs;

    /**
     * The font to use when rendering chat
     */
    private Font chatFont;

    /**
     * Inits the chat utils to accept chat messages
     */
    public ChatUtil() {
        msgs = 0;
        chatBuffer = new ArrayList();
    }

    /**
     * This renders the number of lines of msgs at (x,y) NOTE: for some reason
     * we have to shift the rendering over 20 and down 40
     * 
     * @param g
     *            the graphics context to draw too
     * @param x
     *            the x coord of where to draw
     * @param y
     *            the y coord of where to draw
     * @param lines
     *            the number of lines to render
     */
    public void render(Graphics2D g, int x, int y, int lines) {
        int lns = lines;
        if (msgs != 0) {
            if (lns > MAX_SIZE) {
                lns = MAX_SIZE;
            } else if (lns > chatBuffer.size()) {
                lns = chatBuffer.size();
            }

            int yt = y;
            for (int i = msgs - lns; i < msgs; i++) {
                g.drawString((String) chatBuffer.get(i), x + 10, yt + 15);
                yt += 20;
            }
        }
    }

    /**
     * Adds a msg to the chat buffer
     * 
     * @param msg
     *            the msg to add
     */
    public void addMsg(String msg) {
        if (msgs < MAX_SIZE) {
            chatBuffer.add(msg);
            msgs++;
        } else { //reached the max size
            chatBuffer.remove(0);
            chatBuffer.add(msg);
        }
    }

    /**
     * Gets the chat font so you can use it else where
     * 
     * @return chatFont
     */
    public Font getChatFont() {
        return chatFont;
    }
    
    /**
     * @return Returns the chatBuffer.
     */
    public ArrayList getChatBuffer() {
        return chatBuffer;
    }
    /**
     * @param chatBuffer The chatBuffer to set.
     */
    public void setChatBuffer(ArrayList chatBuffer) {
        this.chatBuffer = chatBuffer;
    }
    /**
     * @return Returns the msgs.
     */
    public int getMsgs() {
        return msgs;
    }
    /**
     * @param msgs The msgs to set.
     */
    public void setMsgs(int msgs) {
        this.msgs = msgs;
    }
    /**
     * @param chatFont The chatFont to set.
     */
    public void setChatFont(Font chatFont) {
        this.chatFont = chatFont;
    }
}