package edu.cs2335.SWATLP.client.gui;

//import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.UnknownHostException;

import edu.cs2335.SWATLP.client.clientdata.PlayerData;

/**
 * The kernel behind the GUI's.
 */
public class MainClass {
    /**
     * DATA: host name to connect to
     */
    private String host = "localhost";

    /**
     * DATA: the port number to use
     */
    private String portNum = "30303";

    /**
     * Connection to the network.
     */
    private ClientConnection connection;

    /**
     * The screenwidth
     */
    private int screenwidth;

    /**
     * the screenheight
     */
    private int screenheight;

    /**
     * The currently displaying window
     */
    private int currentWindow = 0;

    /**
     * Data: success?
     */
    private int success = 0;

    /**
     * DATA: failure?
     */
    private int failure = 1;

    /**
     * is it a new connection?
     */
    private boolean newconnection;

    /**
     * Are they connected?
     */
    private boolean isconnected;

    /**
     * Reference to the main menu gui.
     */
    private MainGui mainmenu;

    /**
     * Reference to the login gui.
     */
    private LoginRender login;

    /**
     * Reference to the character creation gui.
     */
    private CCreateRender ccreate;

    /**
     * Constructor Startup the main menu get the window sizes
     */
    public MainClass() {
        screenwidth = Toolkit.getDefaultToolkit().getScreenSize().width;
        screenheight = Toolkit.getDefaultToolkit().getScreenSize().height;
        mainmenu = new MainGui(this);
    }

    /**
     * Tell if we are connected or not 
     * @return connection status
     */
    public boolean isConnected() {
        return isconnected;
    }
    
    /**
     * Starts the program
     * 
     * @param args
     *            command-line arguments
     */
    public static void main(String[] args) {
        MainClass mc = new MainClass();
        mc.getGui().startRendering();
    }

    /**
     * Displays a chat message to the chat window.
     * 
     * @param txt
     *            the message
     */
    public void displayMainChatMessage(String txt) {
        mainmenu.addMsg(txt);
    }

    /**
     * Logs the player into the server.
     * 
     * @param thehost
     *            the host name of the server
     * @param port
     *            the port number of the server
     * @return Returns a success/failure code.
     */
    public int loginPlayer(String thehost, String port) {
        // set the host and port number
        if (thehost != null) {
            this.host = thehost;
        }
        if (port != null) {
            this.portNum = port;
        }

        // make a new ClientConnection
        int result = openConnection();

        // start the connection
        if (result == 0 && newconnection) {
            Thread connectthread = new Thread(connection);
            connectthread.start();
            newconnection = false;
            isconnected = true;
            System.out.println("Login successful.");
        } else {
            System.out.println("Login failed.");
        }

        return result;
    }

    /**
     * Registers a new player for the server.
     * 
     * @param name
     *            the player's name
     * @param password
     *            password
     * @param ship
     *            the ship type
     * @param skills
     *            the players skills
     * @param homeSolarSystem
     *            home solar system
     * @param governmentType
     *            type of government
     * @param techLevel
     *            level of technology
     * @return Returns a success/failure code.
     */
    public int createPlayer(String name, String password, int[] skills,
            String homeSolarSystem, String governmentType, String techLevel,
            String ship) {
        // check to make sure all fields are filled out
        
        if (name.equals("") || password.equals("") || skills[0] == -5
                || skills[1] == -5 || skills[2] == -5 || skills[3] == -5
                || homeSolarSystem.equals("") || governmentType.equals("")
                || techLevel.equals("") || ship.equals("")) {
            String[] msg = {"All fields are required"};
            mainmenu.displayErrorBox("Empty Fields", msg);

            /*
             * JOptionPane.showMessageDialog(mainmenu, "All fields are
             * required.", "Empty Fields", JOptionPane.INFORMATION_MESSAGE);
             */
            return failure;

            // send the server a NEWPLAYER message w/ the player's info
        } else if (governmentType.equals("Cybernetic")
                && !techLevel.equals("High Tech")
                || governmentType.equals("Feudal Society")
                && !techLevel.equals("Medieval")
                || governmentType.equals("Pacifism")
                && techLevel.equals("High Tech")
                || governmentType.equals("Communism")
                && techLevel.equals("High Tech")) {
            String[] msg = {"Invalid gov tech combination"};
            mainmenu.displayErrorBox("GOV - TECH", msg);

            return failure;
        } else {
            connection.sendMessage("NEWPLAYER~" + name + "~" + password + "~"
                    + skills[1] + "~" + skills[2] + "~" + skills[3] + "~"
                    + skills[0] + "~" + homeSolarSystem + "~" + governmentType
                    + "~" + techLevel + "~" + ship);
            PlayerData data = mainmenu.getPlayerData();
            data.setName(name);
            data.setPass(password);
            mainmenu.setPlayerData(data);
            return success;
        }
    }

    /**
     * Joins the game on the server.
     * 
     * @param name
     *            name of the player
     * @param password
     *            password of the player
     */
    public void startGame(String name, String password) {
        // set the player's name
        connection.setPlayerName(name);

        // send the server a STARTGAME message w/ the player's name/password
        if (name.equals("") && !password.equals("")) {
            String[] msg = {"Please enter a character name and", "password."};
            mainmenu.displayErrorBox("Invalid Character Information", msg);
        } else {
            connection.sendMessage("STARTGAME" + "~" + name + "~" + password);
        }
    }

    /**
     * Gives the MainGui some player data.
     * 
     * @param playerData
     *            "PLAYERDATA", ???
     */
    public void playerData(String[] playerData) {
        // TODO
    }

    /**
     * Gives the player current location in the universe
     * 
     * @param playerLocation blah
     */
    public void playerLoc(String[] playerLocation) {
        if (playerLocation[2].equals("port")) {
            connection.sendMessage("PORTDATA~" + playerLocation[1]);
        } else if (playerLocation[2].equals("lander")) {
            System.out.println("This handlers the luner lander");
            //TODO lander renderer
            //get ship data from the server
        } else if (playerLocation[2].equals("approach")) {
            System.out.println("This handlers the approach");
            //TODO approach stuff
            //get ship data from the server
            //planets data got already
        }
    }

    /**
     * Starts the connection to the server
     * 
     * @return err code
     */
    public int openConnection() {
        try {
            if (connection == null) {
                connection = new ClientConnection(this, host, portNum);
                newconnection = true;
                return success;
            }
        } catch (NumberFormatException n) {
            displayError("SF", "numberformat");
            newconnection = false;
            connection = null;
            return failure;
        } catch (IllegalArgumentException i) {
            displayError("SF", "portrange");
            newconnection = false;
            connection = null;
            return failure;
        } catch (UnknownHostException h) {
            displayError("SF", "host");
            newconnection = false;
            connection = null;
            return failure;
        } catch (IOException h) {
            displayError("SF", "ioerror");
            newconnection = false;
            connection = null;
            return failure;
        }
        return success;
    }

    /**
     * Displays a status message to the user (nonerror).
     * 
     * @param themessage
     *            the message recieved
     */
    public void displayMessage(String themessage) {
        String[] displaymessage = {themessage };
        mainmenu.displayErrorBox("Notice", displaymessage);
    }

    /**
     * Displays an error message
     * 
     * @param titl
     *            the category of the error
     * @param errormessage
     *            the errormessage to display
     */
    public void displayError(String titl, String errormessage) {
        String[] displaymessage = {""};
        String title = null;
        //Component parent = mainmenu; // TODO (was: lobby)
        if (titl == null) {
            title = "Error";
        } else if (titl.equals("SF")) {
            title = "Connection Failure";
        } else if (titl.equals("RF")) {
            title = "Registration Failure";
        } else if (titl.equals("LF")) {
            title = "Login Failure";
        } else if (titl.equals("MTF")) {
            title = "Transaction Failure";
        } else if (titl.equals("LF")) {
            title = "Login Failure";
        } else if (titl.equals("GS")) {
            title = "Action Successful";
            String[] message = {"Gadget successfully purchased."};
            displaymessage = message;
        } else if (titl.equals("MAXSHIELDS")) {
            title = "Action Failed";
            String[] message = {"You already have the maximum",
                    "strength shields available."};
            displaymessage = message;
        } else if (titl.equals("GF")) {
            title = "Action Failed";
            String[] message = {"You already have that gadget."};
            displaymessage = message;
        } else if (titl.equals("LOST")) {
            title = "Game Over";
            if (errormessage.equals("nomoney")) {
                String[] message = {"Your ship has exploded",
                        "You managed to escape in your",
                        "escape pd, but because you had",
                        "no money to buy a new ship,",
                        "the game is over for you."};
                displaymessage = message;
            } else {
            String[] message = {"Due to your inferior ability",
                    "at SWATLP, your life has ended.",
                    "You may start another game",
                    "to begin again."};
            displaymessage = message;
            }
            mainmenu.switchWindow(0);
        } else if (titl.equals("DIED")) {
            title = "Ship Lost";
            String[] message = {"Your ship was lost",
                    "but fortunately you escaped.",
                    "to fight again. You have be",
                    "towed to the nearest port."};
            displaymessage = message;
        } else if (titl.equals("repaired")) {
            title = "Repair Completed Successfully";
            String[] message = {"Your ship has been repaired",
                    "and you have been charged $" + errormessage + "."};
            displaymessage = message;
        } else if (titl.equals("refueled")) {
            title = "Refuel Completed Successfully";
            String[] message = {"Your ship has been refueled",
                    "and you have been charged $" + errormessage + "."};
            displaymessage = message;
        } else if (titl.equals("upgraded")) {
            title = "Shields successfully upgraded";
            String[] message = {"You have been charged $",
                    " " + errormessage + " for the upgrade."};
            displaymessage = message;
        } else {
            title = titl;
        }
        
        if (errormessage == null) {
            String[] message = {"unknown error" };
            displaymessage = message;
        } else {
            if (errormessage.equals("numberformat")) {
                String[] message = {"You have entered an invalid port",
                        "number." };
                displaymessage = message;
            } else if (errormessage.equals("portrange")) {
                String[] message = {"You must specify a port between 1 and",
                        "65535." };

                displaymessage = message;
            } else if (errormessage.equals("host")) {
                String[] message = {"No host found at that address." };
                displaymessage = message;
            } else if (errormessage.equals("ioerror")) {
                String[] message = {"Connection refused by host." };
                displaymessage = message;
            } else if (errormessage.equals("connecterror")) {
                String[] message = {"A connection error has occurred." };
                displaymessage = message;
            } else if (errormessage.equals("nocash")) {
                String[] message = {"You don't have enought cash." };
                displaymessage = message;
            } else if (errormessage.equals("tolittlegood")) {
                String[] message = {"You don't have that much cargo." };
                displaymessage = message;
            } else if (errormessage.equals("nogoods")) {
                String[] message = {"You don't have any cargo of that type." };
                displaymessage = message;
            } else if (errormessage.equals("noroom")) {
                String[] message = {"You don't have enough room for this."};
                displaymessage = message;          
            } else if (errormessage.equals("userexists")) {
                String[] message = {"That username has already\n been " 
                        + "registered."};
                displaymessage = message;               
            } else if (errormessage.equals("planetexists")) {
                String[] message = {"That planet name already exists."};
                displaymessage = message; 
            } else if (errormessage.equals("unknownuser")) {
                String[] message = {"Username not valid. Please try again."};
                displaymessage = message;              
            } else if (errormessage.equals("badpassword")) {
                String[] message = {"You supplied an incorrect password."};
                displaymessage = message;
            } else if (errormessage.equals("loggedon")) {
                String[] message = {"That user is already logged on."};
                displaymessage = message;
            }
        }
        mainmenu.displayErrorBox(title, displaymessage);
    }

    /**
     * this returns a reference to mainGui
     * 
     * @return MainGui
     */
    public MainGui getGui() {
        return mainmenu;
    }

    /**
     *  
     */
    public void shutdown() {
        mainmenu.dispatchEvent(new WindowEvent(mainmenu, 201));
    }

    /**
     * Save the settings to file.
     */
    public final void saveSettings() {
        try {
            File settingsfile = new File("clientSettings.set");
            FileWriter fileWriter = new FileWriter(settingsfile);
            PrintWriter outFile = new PrintWriter(fileWriter);
            outFile.write(host + "\n");

            outFile.write(portNum + "\n");
            outFile.flush();
            outFile.close();
        } catch (IOException e) {
            System.exit(0);
        }
    }

    /**
     * @return The connection of this client using the main class.
     */
    public ClientConnection getConnection() {
        return connection;
    }
    /**
     * @param newconnection The newconnection to set.
     */
    public void setNewconnection(boolean newconnection) {
        this.newconnection = newconnection;
    }
    /**
     * @return Returns the ccreate.
     */
    public CCreateRender getCcreate() {
        return ccreate;
    }
    /**
     * @param ccreate The ccreate to set.
     */
    public void setCcreate(CCreateRender ccreate) {
        this.ccreate = ccreate;
    }
    /**
     * @return Returns the currentWindow.
     */
    public int getCurrentWindow() {
        return currentWindow;
    }
    /**
     * @param currentWindow The currentWindow to set.
     */
    public void setCurrentWindow(int currentWindow) {
        this.currentWindow = currentWindow;
    }
    /**
     * @return Returns the failure.
     */
    public int getFailure() {
        return failure;
    }
    /**
     * @param failure The failure to set.
     */
    public void setFailure(int failure) {
        this.failure = failure;
    }
    /**
     * @return Returns the host.
     */
    public String getHost() {
        return host;
    }
    /**
     * @param host The host to set.
     */
    public void setHost(String host) {
        this.host = host;
    }
    /**
     * @return Returns the isconnected.
     */
    public boolean isIsconnected() {
        return isconnected;
    }
    /**
     * @param isconnected The isconnected to set.
     */
    public void setIsconnected(boolean isconnected) {
        this.isconnected = isconnected;
    }
    /**
     * @return Returns the login.
     */
    public LoginRender getLogin() {
        return login;
    }
    /**
     * @param login The login to set.
     */
    public void setLogin(LoginRender login) {
        this.login = login;
    }
    /**
     * @return Returns the mainmenu.
     */
    public MainGui getMainmenu() {
        return mainmenu;
    }
    /**
     * @param mainmenu The mainmenu to set.
     */
    public void setMainmenu(MainGui mainmenu) {
        this.mainmenu = mainmenu;
    }
    /**
     * @return Returns the portNum.
     */
    public String getPortNum() {
        return portNum;
    }
    /**
     * @param portNum The portNum to set.
     */
    public void setPortNum(String portNum) {
        this.portNum = portNum;
    }
    /**
     * @return Returns the screenheight.
     */
    public int getScreenheight() {
        return screenheight;
    }
    /**
     * @param screenheight The screenheight to set.
     */
    public void setScreenheight(int screenheight) {
        this.screenheight = screenheight;
    }
    /**
     * @return Returns the screenwidth.
     */
    public int getScreenwidth() {
        return screenwidth;
    }
    /**
     * @param screenwidth The screenwidth to set.
     */
    public void setScreenwidth(int screenwidth) {
        this.screenwidth = screenwidth;
    }
    /**
     * @return Returns the success.
     */
    public int getSuccess() {
        return success;
    }
    /**
     * @param success The success to set.
     */
    public void setSuccess(int success) {
        this.success = success;
    }
    /**
     * @return Returns the newconnection.
     */
    public boolean isNewconnection() {
        return newconnection;
    }
    /**
     * @param connection The connection to set.
     */
    public void setConnection(ClientConnection connection) {
        this.connection = connection;
    }
}