
package edu.cs2335.SWATLP.client.gui;

import java.awt.Color;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.Frame;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import edu.cs2335.SWATLP.client.clientdata.ApproachData;
import edu.cs2335.SWATLP.client.clientdata.PlayerData;
import edu.cs2335.SWATLP.client.planets.PlanetAnarchyHighTech;
import edu.cs2335.SWATLP.client.planets.PlanetAnarchyLowTech;
import edu.cs2335.SWATLP.client.planets.PlanetAnarchyMedieval;
import edu.cs2335.SWATLP.client.planets.PlanetCapitalismHighTech;
import edu.cs2335.SWATLP.client.planets.PlanetCapitalismLowTech;
import edu.cs2335.SWATLP.client.planets.PlanetCapitalismMedieval;
import edu.cs2335.SWATLP.client.planets.PlanetCommunismLowTech;
import edu.cs2335.SWATLP.client.planets.PlanetCommunismMedieval;
import edu.cs2335.SWATLP.client.planets.PlanetCyberneticHighTech;
import edu.cs2335.SWATLP.client.planets.PlanetFeudalMedieval;
import edu.cs2335.SWATLP.client.planets.PlanetModel;
import edu.cs2335.SWATLP.client.planets.PlanetPacifismLowTech;
import edu.cs2335.SWATLP.client.planets.PlanetPacifismMedieval;
import edu.cs2335.SWATLP.client.spacegame.PlayerShip;
import edu.cs2335.SWATLP.server.port.TradeGood;

/**
 * @author thaile THis is the main Gui window all other are drawn inside this
 */
public class MainGui extends Frame {
    /** Reference to the control class. */
    private MainClass main;
    /** The currently active window */
    private GuiRender activeWindow;

    /** Refernce to the chat logs */
    private ChatUtil chatlog;
    /** This is the strategy for rendering */
    private BufferStrategy strategy;
    /** This controls the render loop */
    private boolean gameRunning = true;
    /** * This is the standerd font used in the game */
    private Font font;
    
    /** this holds the userName */
    private String userName;

    /** FLAG: display errorBox */
    private boolean displayError = false;
    /** FLAG: display bankbox */
    private boolean displayBank = false;
    /** This is the player data */
    private PlayerData playerData;
    
    /** Comment for zonenum */
    private int zonenum;
    /** the last time */
    private long lasttime;
    /** Comment for currentplanet */
    private PlanetModel currentplanet;
    /** Comment for inzone */
    private boolean inzone;
    
    /** Startup window renderer */
    private StartupRender startwindow;
    /** Login window renderer */
    private LoginRender loginwindow;
    /** Connect window renderer */
    private ConnectRender connectwindow;
    /** Character creation renderer */
    private CCreateRender createwindow;
    /** Port window renderer */
    private PortRender portwindow;
    /** Port window renderer */
    private ChartRender chartwindow;
    /** The approach window renderer */
    private ApproachRender approachwindow;
    /** Market window renderer */
    private MarketRender marketwindow;
    /** Shipyard window renderer */
    private ShipyardRender shipyardwindow;
    /** Error box display */
    private ErrorDialog errorBox;
    /** Bank dialog box */
    private BankDialog bankwindow;
    /** Lunar lander window */
    private LunarRender lunarwindow;
    /** Warehouse window */
    private WarehouseRender warehousewindow;
    /** School window */
    private SchoolRender schoolwindow;    
    /** help window */
    private HelpRender helpwindow;
    
    /** window Constants */
    public static final int START_WINDOW = 0;
    /** window Constants */
    public static final int LOGIN_WINDOW = 1;
    /** window Constants */
    public static final int CONNECT_WINDOW = 2;
    /** window Constants */
    public static final int CREATE_WINDOW = 3;
    /** window Constants */
    public static final int PORT_WINDOW = 4;
    /** window Constants */
    public static final int MARKET_WINDOW = 5;
    /** window Constants */
    public static final int SHIPYARD_WINDOW = 6;
    /** window Constants */
    public static final int APPROACH_WINDOW = 8;
    /** window Constants */
    public static final int CHART_WINDOW = 9;
    /** window Constants */
    public static final int ERROR_WINDOW = 15;
    /** window Constants */
    public static final int BANK_WINDOW = 10;
    /** window constants */
    public static final int LUNAR_WINDOW = 23;
    /** window constants */
    public static final int WAREHOUSE_WINDOW = 16;
    /** window constants */
    public static final int SCHOOL_WINDOW = 12;
    /** window constants */
    public static final int HELP_WINDOW = 13;
    /**
     * This init all the different renderers and set up everything
     * 
     * @param main
     *            MainClass
     */
    public MainGui(MainClass main) {
        super("SWATLP");
        this.main = main;
        setSize(800, 620);
        int screenwidth = Toolkit.getDefaultToolkit().getScreenSize().width;
        int screenheight = Toolkit.getDefaultToolkit().getScreenSize().height;
        setLocation((screenwidth - 800) / 2, (screenheight - 620) / 2);
        setResizable(false);
        inzone = false;
        show();
        createBufferStrategy(2);
        strategy = getBufferStrategy();
        lasttime = System.currentTimeMillis();

        //init
        userName = "test";

        //This basically closes the window
        this.addWindowListener(new WindowStuff());

        //this is so we can contol the tab key
        this.setFocusTraversalKeysEnabled(false);

        chatlog = new ChatUtil();
        
        try {
            File tempfont = new File("fonts/Lucida_Console.ttf");
            FileInputStream readFile = new FileInputStream(tempfont);
            Font tempFont = Font.createFont(Font.TRUETYPE_FONT, readFile);
            font = tempFont.deriveFont(12.0f);
        } catch (FileNotFoundException e) {
            System.out.println("font not found");
        } catch (FontFormatException e) {
            System.out.println("font not formatted");
        } catch (IOException e) {
            System.out.println("error reading in font");
        }

        
        loadImage();
        //this inits and loads all the pics and stuff needed by the renderer
        bankwindow = new BankDialog(this, main);
        startwindow = new StartupRender(this);
        loginwindow = new LoginRender(this, main);
        connectwindow = new ConnectRender(this, main);
        createwindow = new CCreateRender(this, main);
        portwindow = new PortRender(this, main);
        marketwindow = new MarketRender(this, main);
        shipyardwindow = new ShipyardRender(this, main);
        chartwindow = new ChartRender(this, main);
        approachwindow = new ApproachRender(this, main);
        errorBox = new ErrorDialog(this);
        lunarwindow = new LunarRender(this, main);
        warehousewindow = new WarehouseRender(this, main);
        schoolwindow = new SchoolRender(this, main);
        helpwindow = new HelpRender(this, main);
        try {
            FileInputStream fileinput = 
                new FileInputStream("playerData.dat");
            ObjectInputStream objinput = new ObjectInputStream(fileinput);
            playerData = (PlayerData) (objinput.readObject());
            objinput.close();
            fileinput.close();
            loginwindow.setUp(playerData.getHost(), playerData.getPort());
            connectwindow.setUser(playerData.getName(),
                    playerData.getPass());
        } catch (FileNotFoundException e1) {
            //file not found so just use defualt settings
            playerData = new PlayerData();
        } catch (IOException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        } catch (ClassNotFoundException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        
        //TODO change back later
        
        switchWindow(START_WINDOW);
        //testSetupApproach();
        //testLander();
    }

    /**
     *  This load images into memory for the renders
     */
    public final void loadImage() { 
        Toolkit tk = Toolkit.getDefaultToolkit();
        Image b0 = tk.getImage("pics/bg0.jpg");
        Image b1 = tk.getImage("pics/bg1.jpg");
        
        Image but1 = tk.getImage("pics/leftSelect.png");
        Image but7 = tk.getImage("pics/leftSelectarm.png");
        Image but2 = tk.getImage("pics/rightSelect.png");
        Image but8 = tk.getImage("pics/rightSelectarm.png");
        Image but3 = tk.getImage("pics/bbutton.png");
        Image but4 = tk.getImage("pics/bbuttonArm.png");
        Image but5 = tk.getImage("pics/warpB.png");
        Image but6 = tk.getImage("pics/warpBarm.png");
        Image but9 = tk.getImage("pics/done.png");
        Image but10 = tk.getImage("pics/doneArm.PNG");
        Image but11 = tk.getImage("pics/sButton.png");
        Image but12 = tk.getImage("pics/sButtonArm.png");
        Image but13 = tk.getImage("pics/bbuttonL.png");
        Image but14 = tk.getImage("pics/bbuttonLarm.png");
        Image but15 = tk.getImage("pics/leftArrow.png");
        Image but16 = tk.getImage("pics/leftArrowArm.png");
        Image but17 = tk.getImage("pics/rightArrow.png");
        Image but18 = tk.getImage("pics/rightArrowArm.png");
        
        Image sch = tk.getImage("pics/school.jpg");
        Image warp = tk.getImage("pics/SRC.jpg");
        
        Image eb = tk.getImage("pics/errorBox.png");
      //  Image mb = tk.getImage("pics/marketBox.png");
        
        Image demo1 = tk.getImage("pics/shipyardDemo.png");
        Image demo2 = tk.getImage("pics/market.jpg");
        Image demo3 = tk.getImage("pics/star-panorama.jpg");
        Image demo4 = tk.getImage("pics/toolbar.png");
        Image ware = tk.getImage("pics/warehouse.jpg");
        
        Image skill0 = tk.getImage("pics/Combat.png");
        Image skill1 = tk.getImage("pics/Trading.png");
        Image skill2 = tk.getImage("pics/pilot.png");
        Image skill3 = tk.getImage("pics/repair.png");
        
        Image ship1 = tk.getImage("pics/ship/cargo1.png");
        Image ship2 = tk.getImage("pics/ship/fighter1.png");
        Image ship4 = tk.getImage("pics/ship/ultimate1.png");
        Image ship5 = tk.getImage("pics/ship/cargo2.png");
        Image ship6 = tk.getImage("pics/ship/fighter2.png");
        Image ship7 = tk.getImage("pics/ship/ultimate2.png");
        Image ship8 = tk.getImage("pics/ship/cargo3.png");
        Image ship9 = tk.getImage("pics/ship/fighter3.png");
        Image ship10 = tk.getImage("pics/ship/ultimate3.png");
        Image ship3 = tk.getImage("pics/ship_grid.png");
        
        Image approach1 = tk.getImage("pics/approach/asteroidlarge.png");
        Image approach2 = tk.getImage("pics/approach/asteroidmedium.png");
        Image approach3 = tk.getImage("pics/approach/asteroidsmall.png");
        Image ore = tk.getImage("pics/approach/ore.png");
        Image bullet = tk.getImage("pics/approach/bullet.png");
        Image cargo = tk.getImage("pics/approach/cargo1.png");
        Image merchant = tk.getImage("pics/approach/merchant.png");
        Image pirate = tk.getImage("pics/approach/pirate.png");
        Image trader1 = tk.getImage("pics/approach/trader1.png");
        Image police = tk.getImage("pics/ship/police.png");
        Image police2 = tk.getImage("pics/ship/police2.png");
        Image bank = tk.getImage("pics/bank.png");
        
        Image bar = tk.getImage("pics/approach/statusBar.png");

        MediaTracker mt = new MediaTracker(this);
        int i = 1;
        mt.addImage(bar, i++);
        mt.addImage(approach1, i++);
        mt.addImage(approach2, i++);
        mt.addImage(approach3, i++);
        mt.addImage(bullet, i++);
        mt.addImage(cargo, i++);
        mt.addImage(merchant, i++);
        mt.addImage(ore, i++);
        mt.addImage(pirate, i++);
        mt.addImage(trader1, i++);
        mt.addImage(police, i++);
        mt.addImage(police2, i++);
        mt.addImage(b0, i++);
        mt.addImage(b1, i++);
        mt.addImage(bank, i++);
        mt.addImage(sch, i++);
        mt.addImage(but1, i++);
        mt.addImage(but2, i++);
        mt.addImage(but3, i++);
        mt.addImage(but4, i++);
        mt.addImage(but5, i++);
        mt.addImage(but6, i++);
        mt.addImage(but7, i++);
        mt.addImage(but8, i++);
        mt.addImage(but9, i++);
        mt.addImage(but10, i++);
        mt.addImage(but11, i++);
        mt.addImage(but12, i++);
        mt.addImage(but13, i++);
        mt.addImage(but14, i++);
        mt.addImage(but15, i++);
        mt.addImage(but16, i++);
        mt.addImage(but17, i++);
        mt.addImage(but18, i++);
        
        mt.addImage(ware, i++);
        mt.addImage(warp, i++);
        
        mt.addImage(eb, i++);
        
        mt.addImage(demo1, i++);
        mt.addImage(demo2, i++);
        mt.addImage(demo3, i++);
        mt.addImage(demo4, i++);
        
        mt.addImage(skill0, i++);
        mt.addImage(skill1, i++);
        mt.addImage(skill2, i++);
        mt.addImage(skill3, i++);
        
        mt.addImage(ship1, i++);
        mt.addImage(ship2, i++);
        mt.addImage(ship3, i++);
        mt.addImage(ship4, i++);
        
        mt.addImage(ship5, i++);
        mt.addImage(ship6, i++);
        mt.addImage(ship7, i++);
        mt.addImage(ship8, i++);
        mt.addImage(ship9, i++);
        mt.addImage(ship10, i++);
        
        try { //waits til images are loaded
            mt.waitForAll();
        } catch (InterruptedException ie) {
            System.out.println("problem");
        } 
    }
    
    /**
     * this is so the mainClass doesn't lock up
     */
    public void startRendering() {
        renderLoop();
    }

    /**
     * sets the activeWindow param to the window you want
     * 
     * @param newWindow
     *            The window to be displayed
     */
    public final void switchWindow(int newWindow) {
        //remove the old window handlers
        
        this.removeKeyListener(activeWindow);
        this.removeMouseListener(activeWindow);
        this.removeMouseMotionListener(activeWindow);
        
            switch (newWindow) {
            case START_WINDOW:
                activeWindow = startwindow;
                break;
            case LOGIN_WINDOW:
                activeWindow = loginwindow;
                break;
            case CONNECT_WINDOW:
                activeWindow = connectwindow;
                break;
            case CREATE_WINDOW:
                activeWindow = createwindow;
                break;
            case PORT_WINDOW:
                activeWindow = portwindow;
                break;
            case MARKET_WINDOW:
                activeWindow = marketwindow;
                break;
            case SHIPYARD_WINDOW:
                activeWindow = shipyardwindow;
                break;
            case APPROACH_WINDOW:
                activeWindow = approachwindow;
                break;
            case CHART_WINDOW:
                activeWindow = chartwindow;
                break;
            case LUNAR_WINDOW:
                activeWindow = lunarwindow;
                break;
            case WAREHOUSE_WINDOW:
                activeWindow = warehousewindow;
                break;
            case SCHOOL_WINDOW:
                activeWindow = schoolwindow;
                break;
            case HELP_WINDOW:
                activeWindow = helpwindow;
            default:
            }
       
        if (displayError) {
            this.addKeyListener(errorBox);
            this.addMouseListener(errorBox);
            this.addMouseMotionListener(errorBox);
        } else if (displayBank) {
            this.addKeyListener(bankwindow);
            this.addMouseListener(bankwindow);
            this.addMouseMotionListener(bankwindow);
        } else {
            this.addKeyListener(activeWindow);
            this.addMouseListener(activeWindow);
            this.addMouseMotionListener(activeWindow);
        }
        
        
    }

    /**
     * This is a special method to switch the error box off and load the old
     * handlers of the old screen;
     */
    public void closeErrorBox() {
        displayError = false;
        activeWindow = errorBox;
        switchWindow(errorBox.getOldWindow());
    }
    /**
     * This display the error box
     * @param title The title of the box
     * @param msg the msg to be displayed
     */
    public void displayErrorBox(String title, String[] msg) {
        int oldwindow = activeWindow.getWindowNumber();
        errorBox.setText(title, msg);
        errorBox.setOldWindow(oldwindow);
        displayError = true;
        portwindow.setMessagedisplaying(true);
        switchWindow(ERROR_WINDOW);
    }
    
    /**
     * This is a special method to switch the error box off and load the old
     * handlers of the old screen;
     */
    public void closeBankWindow() {
        displayBank = false;
        activeWindow = bankwindow;
        portwindow.setMessagedisplaying(false);
        switchWindow(bankwindow.getOldWindow());
    }
    /**
     * This display the error box
     * @param data the data 
     */
    public void displayBankWindow(String[] data) {
        int oldwindow = activeWindow.getWindowNumber();
        bankwindow.setOldWindow(oldwindow);
        bankwindow.setup(data);
        displayBank = true;
        portwindow.setMessagedisplaying(true);
        switchWindow(BANK_WINDOW);
        
    }

    /**
     * This refreshs the bank data
     * @param data new data
     */
    public void updateBankWindow(String[] data) {
        bankwindow.setup(data);
    }
    
    /**
     * This renders the chat message for us
     * 
     * @param g
     *            the place to draw the chat logs
     * @param x
     *            the x position
     * @param y
     *            the y position
     * @param lines
     *            the number of lines to draw
     */
    public void chatRender(Graphics2D g, int x, int y, int lines) {
        chatlog.render(g, x, y, lines);
    }

    /**
     * @param data        data to update the server with
     * @param currentLoc  the current location of the player
     * @param bankLoan    if they land at the port & have a loan
     */
    public void updateServer(ApproachData data, String currentLoc,
                             boolean bankLoan) {
        String cargo = "";
        for (int n = 0; n < data.getCargo().size(); n++) {
            cargo += ((TradeGood) data.getCargo().get(n)).getGoodtype();
            cargo += "~";
            int amnt = ((TradeGood) data.getCargo().get(n)).getAmount();
            cargo += "" + amnt;
            cargo += "~";
        }
        cargo += "ENDCARGO";
        String loan = "f";
        if (bankLoan) {
            loan = "t";
        }
        main.getConnection().sendMessage("UPDATEDINFO"
                + "~" + cargo
                + "~" + data.getFuelCurrent()
                + "~" + data.getHitpointsCurrent()
                + "~" + data.getReputation()
                + "~" + currentLoc
                + "~" + loan);
    }

    /**
     * This takes in the needed data
     * 
     */
    public void setupPort() {
        portwindow.setupPort(currentplanet.getName());
        switchWindow(PORT_WINDOW);
    }

    /**
     * Setup planet data
     * @param name the planet name
     * @param gov the planet gov
     * @param tech the planet tech
     * @param event the current event on the planet
     * @param x x coord of planet
     * @param y y coord of planet
     */
    public void setPlanetData(String name, String gov, String tech,
            String event, int x, int y) {
        if (gov.equals("Anarchy")) {
            if (tech.equals("High Tech")) {
                currentplanet = new PlanetAnarchyHighTech(name, event, x, y);
            } else if (tech.equals("Low Tech")) {
                currentplanet = new PlanetAnarchyLowTech(name, event, x, y);
            } else if (tech.equals("Medieval")) {
                currentplanet = new PlanetAnarchyMedieval(name, event, x, y);
            }
        } else if (gov.equals("Capitalism")) {
            if (tech.equals("High Tech")) {
                currentplanet = new PlanetCapitalismHighTech(name, event, x,
                        y);
            } else if (tech.equals("Low Tech")) {
                currentplanet = new PlanetCapitalismLowTech(name, event, x, y);
            } else if (tech.equals("Medieval")) {
                currentplanet = new PlanetCapitalismMedieval(name, event, x,
                        y);
            }
        } else if (gov.equals("Communism")) {
            if (tech.equals("Low Tech")) {
                currentplanet = new PlanetCommunismLowTech(name, event, x, y);
            } else if (tech.equals("Medieval")) {
                currentplanet = new PlanetCommunismMedieval(name, event, x, y);
            }
        } else if (gov.equals("Pacifism")) {
            if (tech.equals("Low Tech")) {
                currentplanet = new PlanetPacifismLowTech(name, event, x, y);
            } else if (tech.equals("Medieval")) {
                currentplanet = new PlanetPacifismMedieval(name, event, x, y);
            }
        } else if (gov.equals("Cybernetic")) {
            currentplanet = new PlanetCyberneticHighTech(name, event, x, y);
        } else {
            currentplanet = new PlanetFeudalMedieval(name, event, x, y);
        }
        playerData.visitedPlanet(currentplanet);
    }
    
    
    /**
     * Setup the approach deal.
     * @param somedata  the data
     */
    public void setupApproach(String[] somedata) {
        
        ApproachData data = new ApproachData(somedata, this);
        approachwindow.setupApproachZone(data,
                currentplanet.getGovType());
        switchWindow(APPROACH_WINDOW);
    }
    
    /**
     * This test the approach screen
     *
     */
    public void testSetupApproach() {
        String[] msg = {"APPROACHDATA", "Trader", "0", "0", "ENDCARGO", "1",
                "100", "100", "1", "ENDWEAPONS", "100", "100",
                "0", "0", "0", "0", "0",
                "0", "0", "2", "0", "-2", "6"};
        ApproachData data = new ApproachData(msg, this);
        
        approachwindow.setupApproachZone(data, "Capitalism");
        switchWindow(APPROACH_WINDOW);
    }

    /**
     * @param skills s
     */
    public void setupSchool(String[] skills) {
        schoolwindow.setUp(skills);
        switchWindow(SCHOOL_WINDOW);
    }
    
    
    /**
     * Setup the short range chart.
     * @param data  The data to setup the mofo with.
     */
    public void setupChart(String[] data) {
        chartwindow.setupChart(data);
        switchWindow(CHART_WINDOW);
    }

    /**
     * Setup and switch to the shipyard window.
     * @param shipyarddata  CLIMFOSP!!!
     */
    public void setupShipyard(String[] shipyarddata) {
        // TODO shipyard setup isn't complete
        shipyardwindow.setupShipyard(shipyarddata);
        switchWindow(SHIPYARD_WINDOW);
    }

    /**
     * @param warehousedata w
     */
    public void setupWarehouse(String[] warehousedata) {
        warehousewindow.setup(warehousedata);
        switchWindow(WAREHOUSE_WINDOW);
    }
    
    /**
     * Setup and switch to the market window;
     * @param marketdata  blah
     */
    public void setupMarketplace(String[] marketdata) {
        marketwindow.setup(currentplanet, marketdata);
        switchWindow(MARKET_WINDOW);
    }
    
    /**
     * update the market place after a transaction
     * @param marketdata the data
     */
    public void updateMarketplace(String[] marketdata) {
        System.out.println("update market place");
        marketwindow.setup(currentplanet, marketdata);
        marketwindow.transGood();
    }
    
    /**
     * Adds a chat msg to the chat log
     * 
     * @param msg
     *            the msg to add
     */
    public void addMsg(String msg) {
        chatlog.addMsg(msg);
    }

    /**
     * Get the client username.
     * 
     * @return username
     */
    public String getUserName() {
        return userName;
    }

    /**
     * This sets up the username and pass field when 
     * you create a new user
     * @param username the new username
     * @param pass the new pass field
     */
    public void setUser(String username, String pass) {
        connectwindow.setUser(username, pass);
    }
    
    /**
     * Control the game
     * 
     * @param value
     *            boolean
     */
    public void setGameRunning(boolean value) {
        gameRunning = value;
    }

    /**
     * This method loops til game over
     */
    public void renderLoop() {
        long lastLoopTime = System.currentTimeMillis();

        // keep looping round til the game ends
        while (gameRunning) {
            // work out how long its been since the last update, this
            // will be used to calculate how far the entities should
            // move this loop
            long delta = System.currentTimeMillis() - lastLoopTime;
            lastLoopTime = System.currentTimeMillis();

            // Get hold of a graphics context for the accelerated
            // surface and blank it out
            Graphics2D g = (Graphics2D) strategy.getDrawGraphics();
            g.setColor(Color.black);
            g.fillRect(0, 0, 800, 620);
            g.setFont(font);

            //This calls the proper renderer to draw in the screen
            activeWindow.change(delta);
            activeWindow.render(g);

            //this renders the error box on top of active Screen
            if (displayError) {
                errorBox.render(g);
            } else if (displayBank) {
                bankwindow.render(g);
            }

            // finally, we've completed drawing so clear up the graphics
            // and flip the buffer over
            g.dispose();
            strategy.show();

            long currenttime = System.currentTimeMillis();
            if (currenttime - lasttime < 20) {
                try {
                    Thread.sleep(20 - (currenttime - lasttime)); 
                } catch (InterruptedException e) {
                    break;
                }
            }  
            lasttime = currenttime;
        }
        
        try {
            FileOutputStream fileout = new FileOutputStream("playerData.dat");
            ObjectOutputStream objout = new ObjectOutputStream(fileout);
            objout.writeObject(playerData);
            objout.close();
            fileout.close();
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        
        
        System.exit(0);
    }

    /**
     * handles the window closing
     * 
     * @param e
     *            Event
     * @return if it was handled or not
     */
    public boolean handleEvent(Event e) {
        if (e.id == Event.WINDOW_DESTROY) {
            System.exit(0);
        }
        return true;
    }

    /**
     * window stuff
     */
    private class WindowStuff extends WindowAdapter {
        /**
         * window closing event
         * 
         * @param e
         *            WindowEvent
         */
        public void windowClosed(WindowEvent e) {
            System.out.println("system.save.data.here");
            System.exit(0);
        }

        /**
         * window closing event
         * 
         * @param e
         *            WindowEvent
         */
        public void windowClosing(WindowEvent e) {
            System.out.println("system.save.data.there");
            if (main.isConnected()) {
                System.out.println("closing connection");
                main.getConnection().shutdownClient();
            }
            gameRunning = false;
        }
    }

    /**
     * init the game
     * 
     * @param args
     *            command line args
     */
    public static void main(String[] args) {
        new MainGui(null);
    }
    /**
     * @return Returns the currentplanet.
     */
    public PlanetModel getCurrentplanet() {
        return currentplanet;
    }
    /**
     * @param currentplanet The currentplanet to set.
     */
    public void setCurrentplanet(PlanetModel currentplanet) {
        this.currentplanet = currentplanet;
    }
    
    /**
     * get the player data to edit
     * @return the playerData
     */
    public PlayerData getPlayerData() {
        return playerData;
    }
    /**
     * Set the player data
     * @param playerData the data
     */
    public void setPlayerData(PlayerData playerData) {
        this.playerData = playerData;
        System.out.println("name:" + playerData.getName() 
                        + " pass:" + playerData.getPass()
                        + " host:" + playerData.getHost()
                        + " port:" + playerData.getPort());
    }

    /**
     * @param data d
     * @param player p
     */
    public void setupLander(ApproachData data, PlayerShip player) {
        //this.landerwindow.setupLanderZone(data, player);
        lunarwindow.setupLander(data, player);
        switchWindow(23);
    }
    
    /**
     * This test the approach screen
     *
     *
    public void testLander() {
        ApproachData data = new ApproachData("FIGHTER");
        PlayerShip player = new PlayerShip("FIGHTER");
        
        landerwindow.setupLanderZone(data, player);
        switchWindow(LUNAR_WINDOW);
    }*/
    
    /**
     * @return error box displaying
     */
    public boolean getDispError() {
        return this.displayError;
    }
    /**
     * @return Returns the activeWindow.
     */
    public GuiRender getActiveWindow() {
        return activeWindow;
    }
    /**
     * @param activeWindow The activeWindow to set.
     */
    public void setActiveWindow(GuiRender activeWindow) {
        this.activeWindow = activeWindow;
    }
    /**
     * @return Returns the approachwindow.
     */
    public ApproachRender getApproachwindow() {
        return approachwindow;
    }
    /**
     * @param approachwindow The approachwindow to set.
     */
    public void setApproachwindow(ApproachRender approachwindow) {
        this.approachwindow = approachwindow;
    }
    /**
     * @return Returns the bankwindow.
     */
    public BankDialog getBankwindow() {
        return bankwindow;
    }
    /**
     * @param bankwindow The bankwindow to set.
     */
    public void setBankwindow(BankDialog bankwindow) {
        this.bankwindow = bankwindow;
    }
    /**
     * @return Returns the chartwindow.
     */
    public ChartRender getChartwindow() {
        return chartwindow;
    }
    /**
     * @param chartwindow The chartwindow to set.
     */
    public void setChartwindow(ChartRender chartwindow) {
        this.chartwindow = chartwindow;
    }
    /**
     * @return Returns the chatlog.
     */
    public ChatUtil getChatlog() {
        return chatlog;
    }
    /**
     * @param chatlog The chatlog to set.
     */
    public void setChatlog(ChatUtil chatlog) {
        this.chatlog = chatlog;
    }
    /**
     * @return Returns the connectwindow.
     */
    public ConnectRender getConnectwindow() {
        return connectwindow;
    }
    /**
     * @param connectwindow The connectwindow to set.
     */
    public void setConnectwindow(ConnectRender connectwindow) {
        this.connectwindow = connectwindow;
    }
    /**
     * @return Returns the createwindow.
     */
    public CCreateRender getCreatewindow() {
        return createwindow;
    }
    /**
     * @param createwindow The createwindow to set.
     */
    public void setCreatewindow(CCreateRender createwindow) {
        this.createwindow = createwindow;
    }
    /**
     * @return Returns the displayBank.
     */
    public boolean isDisplayBank() {
        return displayBank;
    }
    /**
     * @param displayBank The displayBank to set.
     */
    public void setDisplayBank(boolean displayBank) {
        this.displayBank = displayBank;
    }
    /**
     * @return Returns the displayError.
     */
    public boolean isDisplayError() {
        return displayError;
    }
    /**
     * @param displayError The displayError to set.
     */
    public void setDisplayError(boolean displayError) {
        this.displayError = displayError;
    }
    /**
     * @return Returns the errorBox.
     */
    public ErrorDialog getErrorBox() {
        return errorBox;
    }
    /**
     * @param errorBox The errorBox to set.
     */
    public void setErrorBox(ErrorDialog errorBox) {
        this.errorBox = errorBox;
    }
    /**
     * @return Returns the font.
     */
    public Font getFont() {
        return font;
    }
    /**
     * @param font The font to set.
     */
    public void setFont(Font font) {
        this.font = font;
    }
    /**
     * @return Returns the inzone.
     */
    public boolean isInzone() {
        return inzone;
    }
    /**
     * @param inzone The inzone to set.
     */
    public void setInzone(boolean inzone) {
        this.inzone = inzone;
    }
    /**
     * @return Returns the lasttime.
     */
    public long getLasttime() {
        return lasttime;
    }
    /**
     * @param lasttime The lasttime to set.
     */
    public void setLasttime(long lasttime) {
        this.lasttime = lasttime;
    }
    /**
     * @return Returns the loginwindow.
     */
    public LoginRender getLoginwindow() {
        return loginwindow;
    }
    /**
     * @param loginwindow The loginwindow to set.
     */
    public void setLoginwindow(LoginRender loginwindow) {
        this.loginwindow = loginwindow;
    }
    /**
     * @return Returns the lunarwindow.
     */
    public LunarRender getLunarwindow() {
        return lunarwindow;
    }
    /**
     * @param lunarwindow The lunarwindow to set.
     */
    public void setLunarwindow(LunarRender lunarwindow) {
        this.lunarwindow = lunarwindow;
    }
    /**
     * @return Returns the main.
     */
    public MainClass getMain() {
        return main;
    }
    /**
     * @param main The main to set.
     */
    public void setMain(MainClass main) {
        this.main = main;
    }
    /**
     * @return Returns the marketwindow.
     */
    public MarketRender getMarketwindow() {
        return marketwindow;
    }
    /**
     * @param marketwindow The marketwindow to set.
     */
    public void setMarketwindow(MarketRender marketwindow) {
        this.marketwindow = marketwindow;
    }
    /**
     * @return Returns the portwindow.
     */
    public PortRender getPortwindow() {
        return portwindow;
    }
    /**
     * @param portwindow The portwindow to set.
     */
    public void setPortwindow(PortRender portwindow) {
        this.portwindow = portwindow;
    }
    /**
     * @return Returns the shipyardwindow.
     */
    public ShipyardRender getShipyardwindow() {
        return shipyardwindow;
    }
    /**
     * @param shipyardwindow The shipyardwindow to set.
     */
    public void setShipyardwindow(ShipyardRender shipyardwindow) {
        this.shipyardwindow = shipyardwindow;
    }
    /**
     * @return Returns the startwindow.
     */
    public StartupRender getStartwindow() {
        return startwindow;
    }
    /**
     * @param startwindow The startwindow to set.
     */
    public void setStartwindow(StartupRender startwindow) {
        this.startwindow = startwindow;
    }
    /**
     * @return Returns the strategy.
     */
    public BufferStrategy getStrategy() {
        return strategy;
    }
    /**
     * @param strategy The strategy to set.
     */
    public void setStrategy(BufferStrategy strategy) {
        this.strategy = strategy;
    }
    /**
     * @return Returns the warehousewindow.
     */
    public WarehouseRender getWarehousewindow() {
        return warehousewindow;
    }
    /**
     * @param warehousewindow The warehousewindow to set.
     */
    public void setWarehousewindow(WarehouseRender warehousewindow) {
        this.warehousewindow = warehousewindow;
    }
    /**
     * @return Returns the zonenum.
     */
    public int getZonenum() {
        return zonenum;
    }
    /**
     * @param zonenum The zonenum to set.
     */
    public void setZonenum(int zonenum) {
        this.zonenum = zonenum;
    }
    /**
     * @return Returns the gameRunning.
     */
    public boolean isGameRunning() {
        return gameRunning;
    }
    /**
     * @param userName The userName to set.
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }
    /**
     * @return Returns the helpwindow.
     */
    public HelpRender getHelpwindow() {
        return helpwindow;
    }
    /**
     * @param helpwindow The helpwindow to set.
     */
    public void setHelpwindow(HelpRender helpwindow) {
        this.helpwindow = helpwindow;
    }
    /**
     * @return Returns the schoolwindow.
     */
    public SchoolRender getSchoolwindow() {
        return schoolwindow;
    }
    /**
     * @param schoolwindow The schoolwindow to set.
     */
    public void setSchoolwindow(SchoolRender schoolwindow) {
        this.schoolwindow = schoolwindow;
    }
}