/*
 * Class  : Georgia Institute of Technology CS2335 Spring 2005  
 * Project: Final
 * File   : StartupRender.java
 * Date   : Apr 6, 2005
 * 
 * Generated by: Scott Le
 *
 * Contributors:
 *           Scott Le
 * 
 */
package edu.cs2335.SWATLP.client.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;

//import com.sun.rsasign.t;

/**
 * Renders the startup window
 */
public class StartupRender extends GuiRender {
    /** this is the reference to the MainGui */
    private MainGui board;

    /** this is the backgroup image */
    private Image bg;
    /** this is the armed pic */
    private Image[] selected = new Image[2];
    /** this is the string of text option */
    private String[] options = {"NEW GAME", "  HELP", "  QUIT"};
    /** the selected option */
    private int selectOpt;
    /** the armed button */
    private int armedOpt;
    
    /** marks the selectedButton */
    private int selectButton;

    /** marks the armed Button */
    private int armButton;
    
    /** 
     * loads and init the data
     * @param main MainGui
     */
    public StartupRender(MainGui main) {
        board = main;

        //Background image
        bg = (new ImageIcon("pics/bg0.jpg")).getImage();
        selected[0] = (new ImageIcon("pics/leftSelect.png")).getImage();
        selected[1] = (new ImageIcon("pics/rightSelect.png")).getImage();
        selectOpt = 0;
        armButton = -1;
    }

    /**
     * Renders the window
     * 
     * @param g
     *            the graphic context
     */
    public void render(Graphics2D g) {
        Font oldFont = g.getFont();
        g.setFont(oldFont.deriveFont(32.0f));
        g.drawImage(bg, 0, 20, null);

        //draw the options texts
        int i = 0;
        for (int y = 470; y < 590; y += 40) {
            if (armButton == i) {
                g.setColor(Color.RED);
            } else {
                g.setColor(Color.GREEN);
            }
            g.drawString(options[i], 100, y);
            i++;
        }
        
        //draw the selected button
        g.drawImage(selected[1], 75, 445 + (selectOpt * 40), null);
        g.drawImage(selected[0], 250, 450 + (selectOpt * 40), null);
        
        
    }

    /**
     * This handle mouse move
     * @param e    MouseEvent
     */
    public void mouseMoved(MouseEvent e) {
        if (e.getX() > 75 && e.getX() < 275) {
            if (e.getY() > 450 && e.getY() < 470) {
                selectOpt = 0;
            } else if (e.getY() > 490 && e.getY() < 510) {
                selectOpt = 1;
            } else if (e.getY() > 530 && e.getY() < 550) {
                selectOpt = 2;
            }
        }
    }

 


    /**
     * This handle mouse pressed
     * @param e  MouseEvent
     */
    public void mousePressed(MouseEvent e) {
        if (e.getX() > 75 && e.getX() < 275) {
            if (e.getY() > 450 && e.getY() < 470) {
                armButton = 0;
            } else if (e.getY() > 490 && e.getY() < 510) {
                armButton = 1;
            } else if (e.getY() > 530 && e.getY() < 550) {
                armButton = 2;
            }
        }
    }

    /**
     * This handles mouse releases
     * @param e MouseEvent
     */
    public void mouseReleased(MouseEvent e) {
        System.out.println("x:" + e.getX() + " Y:" + e.getY());
        if (e.getX() > 75 && e.getX() < 275) {
            if (e.getY() > 450 && e.getY() < 470) {
                if (armButton == 0) {
                    processCmd(0);
                }
                selectOpt = 0;
            } else if (e.getY() > 490 && e.getY() < 510) {
                if (armButton == 1) {
                    processCmd(1);
                }
                selectOpt = 1;
            } else if (e.getY() > 530 && e.getY() < 550) {
                if (armButton == 2) {
                    processCmd(2);
                }
                selectOpt = 2;
            }
        }
        armButton = -1;
    }


    /**
     * handler key presses
     * @param e KeyEvent
     */
    public void keyReleased(KeyEvent e) {
        if (e.getKeyCode() == 9) {
            selectButton =  (selectButton + 1) % 3;
        }
    }
    
    /**
     * Handles keyTyped events
     * @param e KeyEvent
     */
    public void keyTyped(KeyEvent e) {
        if (e.getKeyChar() == '\n' || e.getKeyChar() == ' ') {
            processCmd(selectButton);
        } 
    }
    

    /**
     * Perform an action base on which button was pressed
     * 
     * @param which
     *            button was pressed
     */
    public void processCmd(int which) {
        switch (which) {
        case 0:
            board.switchWindow(1);
            break;
        case 1:
            board.switchWindow(MainGui.HELP_WINDOW);
            break;
        case 2:
            board.dispatchEvent(new WindowEvent(board, 201));
            break;
        default:
        }
    }
    
    /**
     * @return Returns the armButton.
     */
    public int getArmButton() {
        return armButton;
    }
    /**
     * @param armButton The armButton to set.
     */
    public void setArmButton(int armButton) {
        this.armButton = armButton;
    }
    /**
     * @return Returns the armedOpt.
     */
    public int getArmedOpt() {
        return armedOpt;
    }
    /**
     * @param armedOpt The armedOpt to set.
     */
    public void setArmedOpt(int armedOpt) {
        this.armedOpt = armedOpt;
    }
    /**
     * @return Returns the bg.
     */
    public Image getBg() {
        return bg;
    }
    /**
     * @param bg The bg to set.
     */
    public void setBg(Image bg) {
        this.bg = bg;
    }
    /**
     * @return Returns the board.
     */
    public MainGui getBoard() {
        return board;
    }
    /**
     * @param board The board to set.
     */
    public void setBoard(MainGui board) {
        this.board = board;
    }
    /**
     * @return Returns the options.
     */
    public String[] getOptions() {
        return options;
    }
    /**
     * @param options The options to set.
     */
    public void setOptions(String[] options) {
        this.options = options;
    }
    /**
     * @return Returns the selectButton.
     */
    public int getSelectButton() {
        return selectButton;
    }
    /**
     * @param selectButton The selectButton to set.
     */
    public void setSelectButton(int selectButton) {
        this.selectButton = selectButton;
    }
    /**
     * @return Returns the selected.
     */
    public Image[] getSelected() {
        return selected;
    }
    /**
     * @param selected The selected to set.
     */
    public void setSelected(Image[] selected) {
        this.selected = selected;
    }
    /**
     * @return Returns the selectOpt.
     */
    public int getSelectOpt() {
        return selectOpt;
    }
    /**
     * @param selectOpt The selectOpt to set.
     */
    public void setSelectOpt(int selectOpt) {
        this.selectOpt = selectOpt;
    }
}