package edu.cs2335.SWATLP.client.spacegame;

import java.util.List;
import java.util.Random;

import edu.cs2335.SWATLP.client.clientdata.ApproachData;
import edu.cs2335.SWATLP.server.port.TradeGood;

/**
 * @author macguy <p/
 */
public class ZoneEngine implements Runnable {

    /**
     * Comment for <code>lastInput</code>
     */
    private char lastInput;

    /**
     * Comment for <code>lastTime</code>
     */
    private long lastTime;

    /**
     * Comment for <code>zone</code>
     */
    private Zone zone;

    /**
     * Comment for <code>govtype</code>
     */
    private String govtype;

    /**
     * Comment for <code>objects</code>
     */
    private List objects;

    /**
     * Comment for <code>playership</code>
     */
    private PlayerShip playership;

    /**
     * Comment for <code>data</code>
     */
    private ApproachData data;

    /**
     * Comment for <code>player</code>
     */
    private int playerloc;

    /**
     * Comment for <code>running</code>
     */
    private boolean running;

    /**
     * Comment for <code>zonenumber</code>
     */
    private int zonenumber;

    /**
     * Comment for <code>randomizer</code>
     */
    private Random randomizer;

    /** updating ish */
    private boolean updating;

    /**
     * Comment for <code>trading</code>
     */
    private boolean trading;

    /**
     * @param player
     *                  player
     * @param thedata
     *                  data
     * @param zonenumber
     *                  num
     * @param govtype
     *                  gov
     */
    public ZoneEngine(String govtype, PlayerShip player, ApproachData thedata,
            int zonenumber) {
        this.zonenumber = zonenumber;
        this.govtype = govtype;
        playership = player;
        data = thedata;
        zone = ApproachPlanet.approachZone(govtype, zonenumber);
        zone.addToObjects(playership);
        objects = zone.getObjectList();
        lastTime = System.currentTimeMillis();
        randomizer = new Random(System.currentTimeMillis());
        updating = true;
        trading = false;
    }

    /**
     *  
     */
    public void run() {
        running = true;
        while (running) {
            if (!playership.isLiving()) {
                running = false;
                continue;
            }
            for (int x = 0; x < objects.size(); x++) {
                if (objects.get(x) instanceof MerchantShip) {
                    if (((MerchantShip) objects.get(x)).isTrading()) {
                        trading = true;
                    }
                }
            }
            if (updating) {
                if (!trading) {
                    try {
                        moveObjects();
                    } catch (IndexOutOfBoundsException e) {
                        System.err.println("DOUBLE REMOVE AGAIN");
                    }
                }
            }
            long currentTime = System.currentTimeMillis();
            if (currentTime - lastTime < 20) {
                try {
                    Thread.sleep(20 - (currentTime - lastTime));
                } catch (InterruptedException e) {
                    e.printStackTrace();
                    return;
                }
            }
            lastTime = currentTime;
        }
    }

    /**
     *  
     */
    private void moveObjects() {
        if (playership.isTransition()) {
            try {
                Thread.sleep(20);
            } catch (Exception e) {
                return;
            }
            running = false;
            zonenumber++;
            return;
        }
        ScreenObject testobject = null;
        for (int x = 0; x < objects.size(); x++) {
            testobject = ((ScreenObject) objects.get(x));
            checkAI(testobject);
            checkTurning(testobject);
            checkCollisions(testobject);

            if (!(testobject instanceof Bullet)) {
                checkFiring(testobject);
            }

            if (testobject instanceof Asteroid) {
                if (((Asteroid) testobject).isExploding()) {
                    explodeAsteroid((Asteroid) testobject);
                }
            }

            if (testobject instanceof Bullet) {
                if (!testobject.isFiring()) {
                    objects.remove(testobject);
                }
            }
            
            if (testobject instanceof MerchantShip) {
                if (((MerchantShip) testobject).isRunning() 
                        && testobject.getYLoc() < 10) {
                    objects.remove(testobject);
                }
            }

            testobject.tick();
        }
    }

    /**
     * @param testobject
     *                  t
     */
    private void checkAI(ScreenObject testobject) {
        boolean bool = testobject.operateAI(playership);
        if (!bool) {
            System.err.println("AI ERROR");
        }
    }

    /**
     * @param firingobject
     *                  f
     */
    private void fireWeapon(ScreenObject firingobject) {
        ScreenObject bullet = new Bullet();
        bullet.setPosition(firingobject.getXLoc()
                + (firingobject.getScreenWidth() / 2), firingobject.getYLoc()
                + (firingobject.getScreenHeight() / 2));
        double dirfacing = firingobject.getDirFacing();
        bullet.setDirfacing(dirfacing);
        double xspeed = 0;
        double yspeed = 0;
        double bulletspeed = 15;
        if ((dirfacing > 0) && (dirfacing <= Math.PI / 2)) {
            xspeed = Math.cos(dirfacing) * bulletspeed;
            yspeed = -Math.sin(dirfacing) * bulletspeed;
        } else if ((dirfacing > Math.PI / 2) && (dirfacing <= Math.PI)) {
            xspeed = -Math.cos(Math.PI - dirfacing) * bulletspeed;
            yspeed = -Math.sin(Math.PI - dirfacing) * bulletspeed;
        } else if ((dirfacing > Math.PI) && (dirfacing <= 3 * Math.PI / 2)) {
            xspeed = -Math.sin((3 * Math.PI / 2) - dirfacing) * bulletspeed;
            yspeed = Math.cos((3 * Math.PI / 2) - dirfacing) * bulletspeed;
        } else {
            xspeed = Math.cos(2 * Math.PI - dirfacing) * bulletspeed;
            yspeed = Math.sin(2 * Math.PI - dirfacing) * bulletspeed;
        }
        bullet.setXSpeed(xspeed);
        bullet.setYSpeed(yspeed);
        bullet.setPosition(bullet.getXLoc() + 2 * xspeed, bullet.getYLoc() + 2
                * yspeed);
        objects.add(bullet);
    }
    
    /**
     * @param firingobject
     *                  f
     */
    private void fireWeapon2(ScreenObject firingobject) {
        ScreenObject bullet = new Bullet();
        bullet.setPosition(firingobject.getXLoc()
                + (firingobject.getScreenWidth() / 2), firingobject.getYLoc()
                + (firingobject.getScreenHeight() / 2));
        double dirfacing = firingobject.getDirFacing();
        bullet.setDirfacing(dirfacing - Math.PI);
        if (bullet.getDirfacing() < 0) {
            bullet.setDirfacing(dirfacing + Math.PI);
        }
        double xspeed = 0;
        double yspeed = 0;
        double bulletspeed = 15;
        if ((bullet.dirfacing > 0) && (bullet.dirfacing <= Math.PI / 2)) {
            xspeed = Math.cos(bullet.dirfacing) * bulletspeed;
            yspeed = -Math.sin(bullet.dirfacing) * bulletspeed;
        } else if ((bullet.dirfacing > Math.PI / 2) 
                && (bullet.dirfacing <= Math.PI)) {
            xspeed = -Math.cos(Math.PI - bullet.dirfacing) * bulletspeed;
            yspeed = -Math.sin(Math.PI - bullet.dirfacing) * bulletspeed;
        } else if ((bullet.dirfacing > Math.PI) 
                && (bullet.dirfacing <= 3 * Math.PI / 2)) {
            xspeed = -Math.sin((3 * Math.PI / 2) 
                    - bullet.dirfacing) * bulletspeed;
            yspeed = Math.cos((3 * Math.PI / 2) 
                    - bullet.dirfacing) * bulletspeed;
        } else {
            xspeed = Math.cos(2 * Math.PI - bullet.dirfacing) * bulletspeed;
            yspeed = Math.sin(2 * Math.PI - bullet.dirfacing) * bulletspeed;
        }
        bullet.setXSpeed(xspeed);
        bullet.setYSpeed(yspeed);
        bullet.setPosition(bullet.getXLoc() + 2 * xspeed, bullet.getYLoc() + 2
                * yspeed);
        objects.add(bullet);
    }

    /**
     * @param testobject
     *                  t
     */
    private void checkCollisions(ScreenObject testobject) {
        if (testobject instanceof PlayerShip) {
            for (int i = 0; i < objects.size(); i++) {
                ScreenObject collisionobject = (ScreenObject) objects.get(i);
                if (!(collisionobject instanceof PlayerShip)) {
                    if (testobject.checkCollision(collisionobject)) {
                        if (collisionobject instanceof Asteroid
                                || collisionobject instanceof Bullet) {
                            ((PlayerShip) testobject).setColliding(true);
                            ((PlayerShip) testobject).setLastCollided(System
                                    .currentTimeMillis());
                        }
                        if (collisionobject instanceof Asteroid) {
                            Asteroid collidingasteroid 
                                = (Asteroid) collisionobject;
                            collidingasteroid.explode();
                            int damagevalue = 0;
                            if (collidingasteroid.getType().equals("large")) {
                                damagevalue = 5;
                            } else if (collidingasteroid.getType().equals(
                                    "medium")) {
                                damagevalue = 3;
                            } else if (collidingasteroid.getType().equals(
                                    "small")) {
                                damagevalue = 1;
                            } else {
                                damagevalue = 0;
                                addOre();
                                objects.remove(collidingasteroid);
                            }
                            if (System.currentTimeMillis()
                                    - collidingasteroid.getLastcollided() 
                                         > 250) {
                                testobject.takeDamage(damagevalue);
                                collidingasteroid.setLastcollided(System
                                        .currentTimeMillis());
                            }
                        }
                        if (collisionobject instanceof Cargo) {
                            Cargo collidingcargo = (Cargo) collisionobject;
                            TradeGood good = collidingcargo.getGood();
                            addToCargo(good);
                            objects.remove(collidingcargo);
                        }
                        if (collisionobject instanceof Bullet) {
                            int damagevalue = 5;
                            collisionobject.setFiring(false);
                            testobject.takeDamage(damagevalue);
                        }
                    }
                }
            }
        }
        if (testobject instanceof Bullet) {
            for (int i = 0; i < objects.size(); i++) {
                ScreenObject collisionobject = (ScreenObject) objects.get(i);
                if (!(collisionobject instanceof PlayerShip)
                        && !(collisionobject instanceof Bullet)) {
                    if (testobject.checkCollision(collisionobject)) {
                        testobject.setFiring(false);
                        if (collisionobject instanceof Asteroid) {
                            Asteroid collidingasteroid 
                                 = (Asteroid) collisionobject;
                            collidingasteroid.explode();
                            testobject.setFiring(false);
                        }
                        if (collisionobject instanceof PirateShip) {
                            PirateShip pirate = (PirateShip) collisionobject;
                            pirate.takeDamage(7 + data.getCombat() 
                                    + data.getEngineering() / 2);
                            if (pirate.getHitpoints() <= 0) {
                                System.err.println("Pirate went KABOOM!!!");
                                objects.remove(collisionobject);
                            }
                        }
                        if (collisionobject instanceof MerchantShip) {
                            MerchantShip merchant 
                                 = (MerchantShip) collisionobject;
                            merchant.takeDamage(10 + data.getCombat()
                                    + data.getEngineering() / 2);
                            if (playership.getReputation() < 5) {
                                playership.reduceReputation();
                            }
                            if (merchant.getHitpoints() <= 0) {
                                System.err.println("Merchant went KABOOM!!!");
                                explodeMerchantShip(merchant);
                                objects.remove(collisionobject);
                            }
                        }
                        if (collisionobject instanceof PoliceShip) {
                            PoliceShip police = (PoliceShip) collisionobject;
                            police.takeDamage(5 + data.getCombat()
                                    + data.getEngineering() / 2);
                            if (playership.getReputation() < 5) {
                                playership.reduceReputation();
                            }
                            if (police.getHitpoints() <= 0) {
                                System.err.println("Police went KABOOM!!!");
                                objects.remove(collisionobject);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     *
     */
    private void addOre() {
        TradeGood ore = new TradeGood("ORE", 1);
        addToCargo(ore);

    }
    
    /**
     * @param toadd t
     * @return a
     */
    public int addToCargo(TradeGood toadd) {
        TradeGood good = null;
        List cargo = data.getCargo();
        int cargoSize = data.getCargoSize();
        int maxCargo = data.getMaxCargo();
        
        //gets the good that we are adding to
        for (int i = 0; i < cargo.size(); i++) {
            if (((TradeGood) cargo.get(i)).getGoodtype()
                    .equals(toadd.getGoodtype())) {
                good = (TradeGood) cargo.remove(i);
                break;
            }
        }
        //found the good
        System.out.println("cargo:" + (toadd.getAmount() + cargoSize) 
                + " good:" + toadd.getGoodtype() + " max cargo:" + maxCargo);
        
        if (good == null) { //exist so add to existing
            if (toadd.getAmount() + cargoSize <= maxCargo) {
                cargo.add(toadd);
                cargoSize += toadd.getAmount();
                data.setCargo(cargo);
                data.setCargoSize(cargoSize);
            } else {
                System.out.println("full");
                return -1;
            }
        } else { //doesn't exist so add to list
            if (toadd.getAmount() + cargoSize <= maxCargo) {
                int amt = toadd.getAmount();
                toadd.setAmount(good.getAmount() + amt);
                cargo.add(toadd);
                cargoSize += amt;
                data.setCargo(cargo);
                data.setCargoSize(cargoSize);
            } else {
                cargo.add(good); //add it back
                data.setCargo(cargo);
                return -1;
            }
        }

        return 0;
    }

    /**
     * @param testobject
     *                  t
     */
    private void checkFiring(ScreenObject testobject) {
        if (testobject.isFiring() || testobject.hasFired()) {
            if (System.currentTimeMillis() - testobject.getFiringtimer() 
                    >= 333) {
                testobject.setFiringtimer(System.currentTimeMillis());
                if (testobject instanceof PlayerShip) {
                    if (Math.random() < .5) {
                        fireWeapon2(testobject);
                    } else {
                        fireWeapon(testobject);
                    }
                } else {
                    fireWeapon(testobject);
                }
            }
            if (!testobject.isFiring()) {
                testobject.setFired(false);
            }
        }
    }

    /**
     * @param testobject
     *                  t
     */
    private void checkTurning(ScreenObject testobject) {
        if (testobject.isTurningleft()) {
            testobject.turnLeft();
        } else if (testobject.isTurningright()) {
            testobject.turnRight();
        }
    }

    /**
     * @param toexplode
     *                  t
     */
    private void explodeAsteroid(Asteroid toexplode) {
        String type = "";
        if (toexplode.getType().equals("large")) {
            type = "medium";
        } else if (toexplode.getType().equals("medium")) {
            type = "small";
        } else if (toexplode.getType().equals("small")) {
            type = "ore";
        } else {
            //type = "ore";
            return;
        }
        int count = randomizer.nextInt(2) + 2;
      //  int speed = 0;
        for (int i = 0; i < count; i++) {
            Asteroid newasteroid = new Asteroid(type);
            newasteroid.setPosition(toexplode.getXLoc(), toexplode.getYLoc());
            int xneg = randomizer.nextInt(2);
            int yneg = randomizer.nextInt(2);
            double xspeed = 0;
            double yspeed = 0;
            xspeed = Math.random();
            yspeed = Math.random();
            if (xneg == 1) {
                xspeed = -xspeed;
            }
            if (yneg == 1) {
                yspeed = -yspeed;
            }
            newasteroid.setXSpeed(xspeed);
            newasteroid.setYSpeed(yspeed);
            objects.add(newasteroid);
        }
        objects.remove(toexplode);
    }

    /**
     * @param updating
     *                  The updating to set.
     */
    public void setUpdating(boolean updating) {
        this.updating = updating;
    }

    /**
     * @param toexplode
     *                  t
     */
    private void explodeMerchantShip(MerchantShip toexplode) {
        List cargo = toexplode.getCargo();
        for (int n = 0; n < cargo.size(); n++) {
            TradeGood good = (TradeGood) cargo.get(n);
            Cargo acargo = new Cargo(good);
            acargo.setPosition(toexplode.getXLoc(), toexplode.getYLoc());
            int xneg = randomizer.nextInt(2);
            int yneg = randomizer.nextInt(2);
            double xspeed = 0;
            double yspeed = 0;
            xspeed = Math.random();
            yspeed = Math.random();
            if (xneg == 1) {
                xspeed = -xspeed;
            }
            if (yneg == 1) {
                yspeed = -yspeed;
            }
            acargo.setXSpeed(xspeed);
            acargo.setYSpeed(yspeed);
            objects.add(acargo);
        }
        objects.remove(toexplode);
    }

    /**
     * @param totrade
     *                  t
     */
    /*private void trade(MerchantShip totrade) {
        // TODO trade with player
    }*/

    /**
     * @return Returns the zonenumber.
     */
    public int getZonenumber() {
        return zonenumber;
    }

    /**
     * @return Returns the objects.
     */
    public List getZoneObjects() {
        return objects;
    }

    /**
     * @return Returns the running.
     */
    public boolean isRunning() {
        return running;
    }
    /**
     * @return Returns the data.
     */
    public ApproachData getData() {
        return data;
    }
    /**
     * @param data The data to set.
     */
    public void setData(ApproachData data) {
        this.data = data;
    }
    /**
     * @return Returns the govtype.
     */
    public String getGovtype() {
        return govtype;
    }
    /**
     * @param govtype The govtype to set.
     */
    public void setGovtype(String govtype) {
        this.govtype = govtype;
    }
    /**
     * @return Returns the lastInput.
     */
    public char getLastInput() {
        return lastInput;
    }
    /**
     * @param lastInput The lastInput to set.
     */
    public void setLastInput(char lastInput) {
        this.lastInput = lastInput;
    }
    /**
     * @return Returns the lastTime.
     */
    public long getLastTime() {
        return lastTime;
    }
    /**
     * @param lastTime The lastTime to set.
     */
    public void setLastTime(long lastTime) {
        this.lastTime = lastTime;
    }
    /**
     * @return Returns the objects.
     */
    public List getObjects() {
        return objects;
    }
    /**
     * @param objects The objects to set.
     */
    public void setObjects(List objects) {
        this.objects = objects;
    }
    /**
     * @return Returns the playerloc.
     */
    public int getPlayerloc() {
        return playerloc;
    }
    /**
     * @param playerloc The playerloc to set.
     */
    public void setPlayerloc(int playerloc) {
        this.playerloc = playerloc;
    }
    /**
     * @return Returns the playership.
     */
    public PlayerShip getPlayership() {
        return playership;
    }
    /**
     * @param playership The playership to set.
     */
    public void setPlayership(PlayerShip playership) {
        this.playership = playership;
    }
    /**
     * @return Returns the randomizer.
     */
    public Random getRandomizer() {
        return randomizer;
    }
    /**
     * @param randomizer The randomizer to set.
     */
    public void setRandomizer(Random randomizer) {
        this.randomizer = randomizer;
    }
    /**
     * @return Returns the trading.
     */
    public boolean isTrading() {
        return trading;
    }
    /**
     * @param trading The trading to set.
     */
    public void setTrading(boolean trading) {
        this.trading = trading;
    }
    /**
     * @return Returns the zone.
     */
    public Zone getZone() {
        return zone;
    }
    /**
     * @param zone The zone to set.
     */
    public void setZone(Zone zone) {
        this.zone = zone;
    }
    /**
     * @return Returns the updating.
     */
    public boolean isUpdating() {
        return updating;
    }
    /**
     * @param running The running to set.
     */
    public void setRunning(boolean running) {
        this.running = running;
    }
    /**
     * @param zonenumber The zonenumber to set.
     */
    public void setZonenumber(int zonenumber) {
        this.zonenumber = zonenumber;
    }
}