package edu.cs2335.SWATLP.server.main;

import java.io.Serializable;

import edu.cs2335.SWATLP.server.port.Warehouse;

/**
 * SWATLP! - CS2335 Lab8
 * 
 * @version 1.0 <p/>Player - Stores information relating to one player such as
 *          their playername, password, skills, gov't, tech, home planet, etc.
 */
public class Player implements Serializable {
    /** This player's name. */
    private String playerName;

    /** The player's password. */
    private String password;

    /** Is the player in jail? */
    private boolean injail;

    /** The player's fine */
    private int fine;

    /**
     * The level of reputation of the player.
     */
    private int reputation;

    /**
     * The player's bank account.
     */
    private int bankaccount;

    /**
     * The cash that the player is holding
     */
    private int cashonhand;

    /** The player's piloting skill. */
    private int piloting;
    /** The player's engineering skill. */
    private int engineering;
    /** The player's trading skill. */
    private int trading;
    /** The player's combat skill. */
    private int combat;

    /**
     * The player's home solar system.
     */
    private String homePlanet;

    /**
     * The player's current planet.
     */
    private String currentPlanet;

    /** 
     * The player's current location.
     * port, zone1, zone2, zone3, zone4, zone5
     */
    private String currentLocation;

    /**
     * The warehouse of the player.
     */
    private Warehouse warehouse;

    /**
     * The player's ship.
     */
    private Ship ship;

    /**
     * The amount of the loan.
     */
    private int debt;

    /**
     * Constructor that creates a new player with the given player information.
     * new player currentlocation is alway on the port of the home planet
     * 
     * @param playerName
     *            playerName
     * @param password
     *            password
     * @param homePlanet
     *            the player home planet
     * @param shipType
     *            the type of ship
     */
    public Player(String playerName, String password, String homePlanet,
            String shipType) {
        this.playerName = playerName;
        this.password = password;
        this.homePlanet = homePlanet;
        this.ship = new Ship(shipType, this);
        this.currentPlanet = homePlanet;
        this.currentLocation = "port";

        if (shipType.equals("Trader")) {
            cashonhand = 4000;
        } else {
            cashonhand = 2000;
        }
        bankaccount = 0;
        debt = 0;
        warehouse = new Warehouse();
        reputation = 1;
        fine = 0;
        injail = false;
    }

    /**
     * Adds to the player's cashonhand.
     * 
     * @param amount
     *            The amount of money to add.
     */
    public void addMoney(int amount) {
        cashonhand += amount;
    }

    /**
     * Spends the player's money;
     * 
     * @param amount
     *            The amount of money to spend.
     * @return Returns success/failure
     */
    public int spend(int amount) {
        if (cashonhand >= amount) {
            cashonhand -= amount;
            return 0;
        } else if (cashonhand < amount && cashonhand + bankaccount >= amount) {
            bankaccount -= amount - cashonhand;
            cashonhand = 0;
            return 0;
        } else {
            return -1;
        }
    }

    /**
     * Net worth calculator
     * 
     * @return Returns the net worth of the player.
     */
    public int calculateNetWorth() {
        int netWorth = 0;

        netWorth += bankaccount;
        netWorth += cashonhand;
        netWorth += ship.calculateNetWorth();
        netWorth += warehouse.calculateNetWorth();

        return netWorth;
    }

    /**
     * Deposits money into the bank.
     * 
     * @param amount
     *            The amount of money to deposit.
     */
    public void depositMoneyIntoBank(int amount) {
        bankaccount += amount;
        cashonhand -= amount;
    }

    /**
     * Withdraws money from the bank.
     * 
     * @param amount
     *            The amount of money to withdraw.
     * @return Returns failure/success
     */
    public int withdrawMoneyFromBank(int amount) {
        if (bankaccount >= amount) {
            bankaccount -= amount;
            cashonhand += amount;
            return 0;
        } else {
            return -1;
        }
    }

    /**
     * Pays off a debt.
     * @param amt the amt to pay off
     * @return Returns success/failure.
     */
    public int payDebt(int amt) {
        if (cashonhand >= amt) {
            if (amt >= debt) {
                cashonhand -= debt;
                debt = 0;
            } else {
                cashonhand -= amt;
                debt -= amt;
            }
            return 0;
        } else if (cashonhand < amt && cashonhand + bankaccount >= amt) {
            if (amt >= debt) {
                bankaccount -= (debt - cashonhand);
                cashonhand = 0;
                debt = 0;
            } else {
                bankaccount -= (amt - cashonhand);
                cashonhand = 0;
                debt -= amt;
            }
            return 0;
        } else {
            return -1;
        }
    }

    /**
     * @return Returns the player's name.
     */
    public String getPlayerName() {
        return playerName;
    }

    /**
     * @param playerName
     *            The name to set.
     */
    public void setPlayerName(String playerName) {
        this.playerName = playerName;
    }

    /**
     * @return Returns the password.
     */
    public String getPassword() {
        return password;
    }

    /**
     * @param password
     *            The password to set.
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * @return Returns the combat.
     */
    public int getCombat() {
        return combat;
    }

    /**
     * @param combat
     *            The combat to set.
     */
    public void setCombat(int combat) {
        this.combat = combat;
    }

    /**
     * @return Returns the engineering.
     */
    public int getEngineering() {
        return engineering;
    }

    /**
     * @param engineering
     *            The engineering to set.
     */
    public void setEngineering(int engineering) {
        this.engineering = engineering;
    }

    /**
     * @return Returns the homeSolarSystem.
     */
    public String getHomePlanet() {
        return homePlanet;
    }

    /**
     * @param planet
     *            The homeSolarSystem to set.
     */
    public void setHomePlanet(String planet) {
        this.homePlanet = planet;
    }

    /**
     * @return Returns the piloting.
     */
    public int getPiloting() {
        return piloting;
    }

    /**
     * @param piloting
     *            The piloting to set.
     */
    public void setPiloting(int piloting) {
        this.piloting = piloting;
    }

    /**
     * @return Returns the cashonhand.
     */
    public int getCashonhand() {
        return cashonhand;
    }

    /**
     * @param cashonhand
     *            The cashonhand to set.
     */
    public void setCashonhand(int cashonhand) {
        this.cashonhand = cashonhand;
    }

    /**
     * @return Returns the reputation.
     */
    public int getReputation() {
        return reputation;
    }

    /**
     * @param reputation
     *            The reputation to set.
     */
    public void setReputation(int reputation) {
        this.reputation = reputation;
    }

    /**
     * @return Returns the trading.
     */
    public int getTrading() {
        return trading;
    }

    /**
     * @param trading
     *            The trading to set.
     */
    public void setTrading(int trading) {
        this.trading = trading;
    }

    /**
     * @return Returns the currentPort.
     */
    public String getCurrentLocation() {
        return currentLocation;
    }

    /**
     * @param location
     *            The currentPort to set.
     */
    public void setCurrentLocation(String location) {
        this.currentLocation = location;
    }

    /**
     * @return Returns the currentPlanet.
     */
    public String getCurrentPlanet() {
        return currentPlanet;
    }

    /**
     * @param currentPlanet The currentPlanet to set.
     */
    public void setCurrentPlanet(String currentPlanet) {
        this.currentPlanet = currentPlanet;
    }

    /**
     * @return Returns the shipType.
     */
    public Ship getShip() {
        return ship;
    }

    /**
     * @param ship
     *            The ship to be set.
     */
    public void setShip(Ship ship) {
        this.ship = ship;
    }

    /**
     * @return Returns the debt of the player.
     */
    public int getDebt() {
        return debt;
    }

    /**
     * Modifier for the debt.
     * 
     * @param debt
     *            The debt to be set.
     */
    public void setDebt(int debt) {
        this.debt = debt;
    }

    /**
     * @return Returns the bankaccount.
     */
    public int getBankaccount() {
        return bankaccount;
    }

    /**
     * @return Returns the warehouse.
     */
    public Warehouse getWarehouse() {
        return warehouse;
    }

    /**
     * @param warehouse
     *            The warehouse to set.
     */
    public void setWarehouse(Warehouse warehouse) {
        this.warehouse = warehouse;
    }

    /**
     * @param bankaccount
     *            The bankaccount to set.
     */
    public void setBankaccount(int bankaccount) {
        this.bankaccount = bankaccount;
    }

    /**
     * @return Returns the injail.
     */
    public boolean isInjail() {
        return injail;
    }

    /**
     * @param injail
     *            The injail to set.
     */
    public void setInjail(boolean injail) {
        this.injail = injail;
    }

    /**
     * @return Returns the fine.
     */
    public int getFine() {
        return fine;
    }

    /**
     * @param fine
     *            The fine to set.
     */
    public void setFine(int fine) {
        this.fine = fine;
    }
}