package edu.cs2335.SWATLP.server.main;

import java.io.Serializable;
import java.util.Hashtable;

import edu.cs2335.SWATLP.server.port.Warehouse;

/**
 * SWATLP! - CS2335 Lab8
 * 
 * @version 1.0 <p/>PlayerDatabase - Stores all the player objects and player
 *          information.
 */
public class PlayerDatabase implements Serializable {
    /**
     * Success code.
     */
    private static int success = 1;

    /**
     * Code to indicate an invalid player was specified by some action.
     */
    private static int noplayer = 2;

    /**
     * Code to indicate the given password was invalid.
     */
    private static int badpass = 3;

    /**
     * Code to indicate another player with the same name was found.
     */
    private static int duplicateplayer = 4;

    /**
     * A list of all player objects.
     */
    private Hashtable playerList;

    /** the universe */
    private Universe gameworld;

    /**
     * Constructor that initializes the playerList.
     */
    public PlayerDatabase() {
        playerList = new Hashtable();
    }

    /**
     * Registers a new player with the given information.
     * 
     * @param playerName
     *            the player's name
     * @param password
     *            the player's password
     * @param skills
     *            the player's skills
     * @param homePlanet
     *            the player's home planet
     * @param ship
     *            the player's shiptype
     * @return duplicateplayer: a player with that name already exists success:
     *         player registered successfully
     */
    public int createPlayer(String playerName, String password,
            int[] skills, String homePlanet, String ship) {
        // search for a duplicate player's name
        if (playerList.get(playerName) != null) {
            return duplicateplayer;
        }

        // add the player to the database
        Player newplayer = new Player(playerName, password, homePlanet, ship);
        newplayer.setPiloting(skills[0]);
        newplayer.setEngineering(skills[1]);
        newplayer.setTrading(skills[2]);
        newplayer.setCombat(skills[3]);
        newplayer.setWarehouse(new Warehouse());
        playerList.put(playerName, newplayer);

        return success;
    }

    /**
     * Logs a player in to the system if they provided a valid player name and
     * password.
     * 
     * @param playerName
     *            the login name
     * @param password
     *            the password
     * @return success login successful badpass invalid password noplayer
     *         invalid player name
     */
    public int loginPlayer(String playerName, String password) {
        System.out.println("checking for player " + playerName);
        if (playerList.get(playerName) != null) {
            Player data = (Player) playerList.get(playerName);
            if (data.getPassword().equals(password)) {
                return success;
            } else {
                return badpass;
            }
        }
        return noplayer;
    }

    /**
     * Returns the player object with the playerName as given.
     * 
     * @param playerName
     *            the name of the player to get
     * @return currentPlayer the player null no player with that name found
     */
    public Player getPlayer(String playerName) {
        return (Player) playerList.get(playerName);
    }

    /**
     * Do we need this for anything?
     * 
     * @return Returns the list of players.
     */
    public Hashtable getPlayerList() {
        return playerList;
    }
    /**
     * @return Returns the gameworld.
     */
    public Universe getGameworld() {
        return gameworld;
    }
    /**
     * @param gameworld The gameworld to set.
     */
    public void setGameworld(Universe gameworld) {
        this.gameworld = gameworld;
    }
    /**
     * @param playerList The playerList to set.
     */
    public void setPlayerList(Hashtable playerList) {
        this.playerList = playerList;
    }
}