package edu.cs2335.SWATLP.server.main;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import edu.cs2335.SWATLP.server.port.Bank;
import edu.cs2335.SWATLP.server.port.Shipyard;
import edu.cs2335.SWATLP.server.port.TradeGood;

/**
 * SWATLP! - CS2335 Lab8
 * 
 * @version 1.0 <p/>ServerConnection - The server connection class. Handles one
 *          connection to one client and transfers data to and from them.
 */
public class ServerConnection implements Runnable {
    /**
     * Success code.
     */
    private static int success = 1;

    /**
     * Code to indicate an invalid user was specified by some action.
     */
    private static int noplayer = 2;

    /**
     * Code to indicate an invalid password was entered by the user.
     */
    private static int badpass = 3;

    /**
     * Code to indicate a username specified by some action already exists.
     */
    private static int duplicateplayer = 4;

    /**
     * The client socket.
     */
    private Socket connection;

    /**
     * The main server class.
     */
    private Server server;

    /**
     * Input stream to read text from client.
     */
    private BufferedReader input;

    /**
     * Output stream to write text to client.
     */
    private PrintWriter output;

    /**
     * The playerName of the connected user associated with this connection.
     */
    private String playerName;

    /**
     * A message received from the client.
     */
    private String[] message;

    /**
     * A reference to the user database.
     */
    private PlayerDatabase database;

    /**
     * A referense to the player.
     */
    private Player player;

    /**
     * Input stream to read objects from client.
     */
    private ObjectInputStream objectInput;

    /**
     * Output stream to write objects to server.
     */
    private ObjectOutputStream objectOutput;

    /**
     * Name of the game the user is in.
     */
    private String gameName;

    /**
     * A boolean that controls whether or not the thread continues at the start
     * of each loop.
     */
    private boolean run;

    /**
     * Constructor which is called from the main server class when a new
     * connection is accepted.
     * 
     * @param connection
     *            the client socket
     * @param server
     *            an instance of the main server class
     * @param database
     *            a refernce to the user database
     */
    public ServerConnection(Socket connection, Server server,
            PlayerDatabase database) {
        this.database = database;
        this.connection = connection;
        this.server = server;
        playerName = "no_one";
    }

    /**
     * Main thread loop. Keeps looping and accepting messages until client
     * quits.
     */
    public void run() {
        run = true;
        try {
            input = new BufferedReader(new InputStreamReader(connection
                    .getInputStream()));
            output = new PrintWriter(connection.getOutputStream());
        } catch (IOException e) {
            e.printStackTrace();
        }

        while (run) {
            try {
                String capturedmessage = input.readLine();
                if (capturedmessage == null) {
                    run = false;
                    continue;
                }
                message = capturedmessage.split("~");
                if (message == null) {
                    run = false;
                    continue;
                }
                parseMessage(message);
            } catch (SocketException se) {
                System.out.println("client left");
                closeConnection();
            } catch (IOException e) {
                e.printStackTrace();
                closeConnection();
            }
        }
    }

    /**
     * Parses the command from the connected user and determines what action to
     * take based on that.
     * 
     * @param text
     *            the text received from the client
     */
    private void parseMessage(String[] text) {

        // client sent: a chat message to display to everyone
        if (text[0].equals("MAINMSG")) {
            System.out.println("recieved chat msg");
            sendMainChatMsg(text);

            // client sent: a request to begin/join a game
        } else if (text[0].equals("STARTGAME")) {
            startGame(text);

            // client sent: new character information
        } else if (text[0].equals("NEWPLAYER")) {
            createPlayer(text);
            // client sent: they want some Player data
        } else if (text[0].equals("PLAYERMONEY")) {
            sendPlayerFinance(text);
            // client sent: they want the player location
        } else if (text[0].equals("PLAYERLOCATION")) {
            sendPlayerLoc(text);
            // client sent: they want to quit
        } else if (text[0].equals("QUIT")) {
            System.out.println("Closing connection for " + playerName);
            closeConnection();
        } else if (text[0].equals("MARKETDATA")) {
            sendMarketData(text);
        } else if (text[0].equals("SHIPYARDDATA")) {
            sendShipYardData(text);
        } else if (text[0].equals("SHIPYARDTRANS")) {
            getShipyardData(text);
        } else if (text[0].equals("BANKDATA")) {
            sendBankData();
        } else if (text[0].equals("BANKTRANS")) {
            getBankTrans(text);
        } else if (text[0].equals("SCHOOLDATA")) {
            sendSchoolData(text);
        } else if (text[0].equals("RANGECHARTDATA")) {
            sendRangeData(text);
        } else if (text[0].equals("MARKETTRANS")) {
            doTransaction(text);
        } else if (text[0].equals("APPROACHDATA")) {
            sendApproachData(text);
        } else if (text[0].equals("PLANETDATA")) {
            sendPlanetData(text);
        } else if (text[0].equals("PORTDATA")) {
            sendPortData(text);
        } else if (text[0].equals("SHIPLIST")) {
            sendShipList(text);
        } else if (text[0].equals("UPDATEDINFO")) {
            getUpdatedInfo(text);
        } else if (text[0].equals("WAREHOUSEDATA")) {
            sendWarehousedata(text);
        } else if (text[0].equals("WAREHOUSETRANS")) {
            processWarehouseTrans(text);
        } else {
            sendMessage("X~UF~unknownerror");
        }
    }

    /**
     * update you cargo and warehouse with the new data
     * @param data d
     */
    public void processWarehouseTrans(String[] data) {

        Player aplayer = database.getPlayer(playerName);
        
        aplayer.getShip().setCargoSize(Integer.parseInt(data[1]));
        List cargo = new ArrayList();
        
        for (int i = 0; i < 8; i++) {
            if (!data[(i * 2) + 3].equals("0")) {
                System.out.println("found wares:" + data[(i * 2) + 2]);
                cargo.add(new TradeGood(data[(i * 2) + 2],
                        Integer.parseInt(data[(i * 2) + 3])));
            }
        }
        aplayer.getShip().setCargo(cargo);
        List wares = new ArrayList();
        for (int i = 0; i < 8; i++) {
            if (!data[(i * 2) + 19].equals("0")) {
                System.out.println("found wares:" + data[(i * 2) + 18]);
                wares.add(new TradeGood(data[(i * 2) + 18],
                        Integer.parseInt(data[(i * 2) + 19])));
            }
        }
        aplayer.getWarehouse().setTradegoods(wares);
    }
    
    /**
     * Get the warehouse data
     * @param data not used yet 
     */
    public void sendWarehousedata(String[] data) {
        Player aplayer = database.getPlayer(playerName);
        String msg = "WAREHOUSEDATA~" + aplayer.getShip().getCargoSize() 
            + "~" + aplayer.getShip().getMaxCargo() + "~";

        List cargo = aplayer.getShip().getCargo();
        msg += cargo.size() + "~";
        for (int i = 0; i < cargo.size(); i++) {
            TradeGood good = (TradeGood) cargo.get(i);
            msg += good.getGoodtype() + "~" + good.getAmount() + "~";
        }
        
        msg += aplayer.getWarehouse().displayContents();
        
        sendMessage(msg);
        
        
    }
    
    /**
     * This preforms the needed transactions or returns an error
     * @param cmds  MARKETTRANS~buy/sell~good~amt~total
     */
    public void doTransaction(String[] cmds) {
        String msg = "X~MTF";
        Player data = database.getPlayer(playerName);
        Ship ship = data.getShip();
        int total = Integer.parseInt(cmds[4]); 
        TradeGood good = new TradeGood(cmds[2], Integer.parseInt(cmds[3]));
        
        if (cmds[1].equals("BUY")) {
            int result = data.spend(total);
            if (result == 0) { 
                int add = ship.addToCargo(good);
                if (add == 0) {
                    msg = "MARKETTRANS~" + data.getTrading() + "~" 
                    + data.getBankaccount() + "~" + data.getCashonhand() + "~" 
                    + data.getShip().getMaxCargo() + "~";
                    
                    List cargo = data.getShip().getCargo();
                    msg += "" + cargo.size() + "~";
                    for (int i = 0; i < cargo.size(); i++) {
                        TradeGood temp = (TradeGood) cargo.get(i);
                        msg += temp.getGoodtype() + "~"  
                             + temp.getAmount() + "~";
                    }
                } else {
                    data.addMoney(total); //replace what was taken out
                    msg = "X~MTF~noroom";
                }
            } else {  //not enought cash
                msg = "X~MTF~nocash";
            }
        } else if (cmds[1].equals("SELL")) { //sell
            int result = ship.removeCargo(good);
            switch(result) {
            case 0: //good
                data.addMoney(total);
                msg = "MARKETTRANS~" + data.getTrading() + "~"
                + data.getBankaccount() + "~" + data.getCashonhand() + "~" 
                + data.getShip().getMaxCargo() + "~";

                List cargo = data.getShip().getCargo();
                msg += "" + cargo.size() + "~";
                for (int i = 0; i < cargo.size(); i++) {
                    TradeGood temp = (TradeGood) cargo.get(i);
                    msg += temp.getGoodtype() + "~" + temp.getAmount() + "~";
                }
                break;
            case -1:
                msg = "X~MTF~tolittlegood";
                break;
            case -2:
                msg = "X~MTF~nogoods";
                break;
            default:
            }
           
        } 
        sendMessage(msg);
    }
    
    /**
     * Requests from the server some shipyard data.
     * 
     * @param cmd SHIPYARDDATA
     */
    public void sendShipYardData(String[] cmd) {
        Player aplayer = database.getPlayer(playerName);

        String msg = "SHIPYARDDATA~" + aplayer.getBankaccount() + "~"
                + aplayer.getCashonhand() + "~";

        msg += aplayer.getShip().toString();

        sendMessage(msg);
    }
    /**
     * @param data shipyard stuff
     */
    public void getShipyardData(String[] data) {
        Player aplayer = database.getPlayer(playerName);

        boolean transfer = false;

        if (data[1].equals("buyship")) {
            if (data[3].equals("yes")) {
                transfer = true;
            }
            int result = Shipyard.buyShip(aplayer, data[2], transfer);
            if (result == 0) {
                sendShipYardData(data);
                return;
            }
        } else if (data[1].equals("buygadget")) {
            int result = Shipyard.buyGadget(aplayer, data[2]);
            if (result == 0) {
                sendShipYardData(data);
                sendMessage("X~GS");
                return;
            } else if (result == -1) {
                sendMessage("X~GF");
                return;
            }
        } else if (data[1].equals("buyweapon")) {
            int result = Shipyard.buyWeapon(aplayer, data[2]);
            if (result == 0) {
                sendShipYardData(data);
                return;
            }
        } else if (data[1].equals("repairship")) {
            int cost = Shipyard.repairShip(aplayer);
            if (cost != -1) {
                sendMessage("SHIPYARDTRANS~repaired~" + cost);
                sendShipYardData(data);
                return;
            }
        } else if (data[1].equals("refuelship")) {
            int cost = Shipyard.refuelShip(aplayer);
            if (cost != -1) {
                sendMessage("SHIPYARDTRANS~refueled~" + cost);
                sendShipYardData(data);
                return;
            }
        } else if (data[1].equals("upgradeshields")) {
            int result = Shipyard.upgradeShields(aplayer);
            if (result > 0) {
                sendMessage("X~upgraded~" + result);
                sendShipYardData(data);
                return;
            } else if (result == -2) {
                sendMessage("X~MAXSHIELDS");
                return;
            }
        } else if (data[1].equals("reorderweapon")) {
            Shipyard.reorderWeapon(aplayer, Integer.parseInt(data[2]));
            sendShipYardData(data);
        } else if (data[1].equals("removeweapon")) {
            int result = Shipyard.removeWeapon(aplayer, data[2]);
            if (result == 0) {
                sendShipYardData(data);
            } else {
                sendMessage("X~UF~unknownerror");
            }
        }
        sendMessage("X~MTF~nocash");
    }

    /**
     * Requests from the server some bank data.
     */
    public void sendBankData() {
        Player aplayer = database.getPlayer(playerName);

        String msg = "BANKDATA~" + aplayer.getBankaccount() + "~"
        + aplayer.getCashonhand() + "~" + aplayer.getDebt() + "~"
        + aplayer.calculateNetWorth() + "~";

        if (Bank.canGetLoan(aplayer)) {
            msg += "t";
        } else {
            msg += "f";
        }

        sendMessage(msg);
    }
    
    /**
     * @param msg m
     */
    public void sendShipList(String[] msg) {
        
    }

    /**
     * Sends server some bank transaction data, server sends back bank data.
     * @param cmd  protocol
     */
    public void getBankTrans(String[] cmd) {
        Player aplayer = database.getPlayer(playerName);

        if (cmd[1].equals("deposit")) {
            Bank.deposit(aplayer, Integer.parseInt(cmd[2]));
        } else if (cmd[1].equals("withdraw")) {
            Bank.withdraw(aplayer, Integer.parseInt(cmd[2]));
        } else if (cmd[1].equals("paydebt")) {
            Bank.payDebt(aplayer, Integer.parseInt(cmd[2]));
        } else if (cmd[1].equals("loan")) {
            Bank.takeALoan(aplayer, aplayer.calculateNetWorth());
        }

        sendBankData();
    }

    /**
     * Requests from the server some school data.
     * 
     * @param cmd
     *            The string protocol
     */
    public void sendSchoolData(String[] cmd) {
        Player aplayer = database.getPlayer(playerName);
        sendMessage("SCHOOLDATA~" + aplayer.getCashonhand() + "~" 
                + aplayer.getPiloting() + "~" + aplayer.getEngineering() 
                + "~" + aplayer.getTrading() + "~" + aplayer.getCombat());
        
    }

    /**
     * Requests from the server some port data.
     * 
     * @param cmd
     *            The string protocol
     */
    public void sendPortData(String[] cmd) {
        // get the planet
        Planet playerPlanet = server.getGameworld().getPlanet(cmd[1]);

        // send the client the planet data
        if (playerPlanet != null) {
            sendMessage("PORTDATA" + "~" + playerPlanet.getName() + "~"
                    + playerPlanet.getGovType() + "~"
                    + playerPlanet.getTechType() + "~"
                    + playerPlanet.getEvent() + "~"
                    + playerPlanet.getXlocation() + "~"
                    + playerPlanet.getYlocation());
        }
    }

    /**
     * Requests from the server some port data.
     * 
     * @param cmd
     *            The string protocol
     */
    public void sendPlanetData(String[] cmd) {
        // get the planet
        Planet playerPlanet = server.getGameworld().getPlanet(cmd[1]);

        String joining = "f";
        if (cmd[2].equals("t")) {
            joining = "t";
        }

        // send the client the planet data
        if (playerPlanet != null) {
            sendMessage("PLANETDATA" + "~" + playerPlanet.getName() + "~"
                    + playerPlanet.getGovType() + "~"
                    + playerPlanet.getTechType() + "~"
                    + playerPlanet.getEvent() + "~"
                    + playerPlanet.getXlocation() + "~"
                    + playerPlanet.getYlocation() + "~"
                    + joining);
        }
    }

    /**
     * Requests from the server some market data.
     * 
     * @param cmd
     *            The string protocol
     */
    public void sendMarketData(String[] cmd) {
        Player aplayer = database.getPlayer(playerName);

        String msg = "MARKETDATA~" + aplayer.getTrading() + "~"
                + aplayer.getBankaccount() + "~" + aplayer.getCashonhand()
                + "~" + aplayer.getShip().getMaxCargo() + "~";

        List cargo = aplayer.getShip().getCargo();
        msg += "" + cargo.size() + "~";
        for (int i = 0; i < cargo.size(); i++) {
            TradeGood good = (TradeGood) cargo.get(i);
            msg += good.getGoodtype() + "~" + good.getAmount() + "~";
        }
        sendMessage(msg);
    }

    /**
     * Sends a chat message from this client to the chat window of all all
     * connected clients.
     * 
     * @param imessage
     *            the message to send
     */
    private void sendMainChatMsg(String[] imessage) {
        System.out.println("SERVERCON: msg:" + imessage[1]);
        if (imessage.length != 2) {
            sendMessage("X~UF~unknownerror");
            return;
        }
        String msgtosend = imessage[1];
        server.sendMessage("MAINMSG~" + playerName + "~" + msgtosend);
    }

    /**
     * Authenticates the client with the user database and logs them in if the
     * information provided is valid.
     * 
     * @param loginstring
     *            the login command received
     */
    private void startGame(String[] loginstring) {
        System.out.println("Starting game");
        if (loginstring.length != 3) {
            sendMessage("X~UF~unknownerror");
            return;
        }

        // connect the player to the game
        String playername = loginstring[1];
        String password = loginstring[2];
        if (server.getPlayerConnection(playername) == null) {
            int result = database.loginPlayer(playername, password);
            if (result == success) {
                player = database.getPlayer(playername);
                this.playerName = playername;
                if (player.getCurrentLocation().equals("port")) {
                    String msg = "PORTDATA~" + player.getCurrentPlanet();
                    String[] blah = msg.split("~");
                    sendPortData(blah);
                } else if (player.getCurrentLocation() != null) {
                    String msg = "PLANETDATA~" + player.getCurrentPlanet()
                               + "~t";
                    String[] blah = msg.split("~");
                    sendPlanetData(blah);
                }
            } else if (result == noplayer) {
                System.out.println("no player");
                sendMessage("X~LF~unknownuser");
            } else if (result == badpass) {
                sendMessage("X~LF~badpassword");
            }
        } else {
            sendMessage("X~LF~loggedon");
            player = database.getPlayer(playerName);
        }
    }

    /**
     * Registers a new username and password in the user database.
     * 
     * @param playerString
     *            the registration info received
     */
    private void createPlayer(String[] playerString) {
        System.out.println("Creating new player");
        if (playerString.length != 11) {
            sendMessage("X~UF~unknownerror");
            return;
        }

        // fill up some data
        String playername = playerString[1];
        String password = playerString[2];
        int[] skills = new int[4];
        skills[0] = Integer.parseInt(playerString[3]);
        skills[1] = Integer.parseInt(playerString[4]);
        skills[2] = Integer.parseInt(playerString[5]);
        skills[3] = Integer.parseInt(playerString[6]);
        String homePlanet = playerString[7];
        String governmentType = playerString[8];
        String techLevel = playerString[9];
        String ship = playerString[10];
        // add the player to the database
        int result = database.createPlayer(playername, password, skills,
                homePlanet, ship);
        boolean addedPlanet = server.getGameworld().addPlanet(homePlanet,
                governmentType, techLevel);
        System.out.println(addedPlanet);
        if (result == success && addedPlanet) {
            sendMessage("RS~" + playername + "~" + password 
                    + "~" + homePlanet);
        } else if (result == duplicateplayer) {
            sendMessage("X~RF~userexists");
        } else if (!addedPlanet) {
            sendMessage("X~RF~planetexists");
        }
    }

    /**
     * Requests from the server some range data.
     * 
     * @param dataString  RANGECHARTDATA
     */
    public void sendRangeData(String[] dataString) {
        Player aplayer = database.getPlayer(playerName);
        Ship aship = aplayer.getShip();
        
        String planetinfo = "RANGECHARTDATA" + "~" 
            + aplayer.getShip().getFuelCurrent() + "~";
        List planetsinrange = server.getGameworld().getPlanetsInRange(aship);
        for (int i = 0; i < planetsinrange.size(); i++) {
            planetinfo += ((Planet) planetsinrange.get(i)).getName() + "~";
            planetinfo += "" + ((Planet) planetsinrange.get(i)).getXlocation()
                            + "~";
            planetinfo += "" + ((Planet) planetsinrange.get(i)).getYlocation()
                            + "~";
            //planetinfo += ((Planet) planetsinrange.get(i)).getGovType() + "~";
           //planetinfo += ((Planet) planetsinrange.get(i)).getTechType() + "~";
        }
        planetinfo += "ENDLIST";

        //String playerinfo = "";
        //playerinfo += "" + aship.getX() + "~" + "" + aship.getY();
        sendMessage(planetinfo); // + "~" + playerinfo);
    }

    /**
     * Requests from the server the player finance data.
     * 
     * @param dataString
     *            [0] = "PLAYERDATA", [1] = player name
     */
    public void sendPlayerFinance(String[] dataString) {
        // get the player
        Player aplayer = database.getPlayer(dataString[1]);

        // send the client the player data
        if (aplayer != null) {
            // TODO
            return;
        }
    }

    /**
     * Requests from the server a player location.
     * 
     * @param dataString
     *            [0] = "PLAYERLOC", [1] = player loc
     */
    public void sendPlayerLoc(String[] dataString) {
        // get the player
        Player aplayer = database.getPlayer(dataString[1]);

        // send the client the player data
        if (aplayer != null) {
            sendMessage("PLAYERLOC" + "~" + aplayer.getCurrentLocation());
        }
    }

    /**
     * Sends to the client some combatdata.
     * @param data          the planet name
     */
    public void sendApproachData(String[] data) {
        Player aplayer = database.getPlayer(playerName);
        Ship ship = aplayer.getShip();
        Planet oldPlanet = server.getGameworld().getPlanet(
                player.getCurrentPlanet());
        Planet planet = server.getGameworld().getPlanet(data[1]);

        // NOTE: this shit sets the planet to the new planet, could be put
        //       elsewhere
        player.setCurrentPlanet(planet.getName());

        int cargosize = 0;
        String cargo = "";
        for (int n = 0; n < ship.getCargo().size(); n++) {
            cargo += ((TradeGood) ship.getCargo().get(n)).getGoodtype();
            cargo += "~";
            int amnt = ((TradeGood) ship.getCargo().get(n)).getAmount();
            cargosize += amnt;
            cargo += "" + amnt;
            cargo += "~";
        }
        cargo += "ENDCARGO";
        String weapons = "";
        for (int n = 0; n < ship.getWeapons().size(); n++) {
            weapons += ((Weapon) ship.getWeapons().get(n)).getName();
            weapons += "~";
        }
        weapons += "ENDWEAPONS";
        String gadgets = "";
        for (int n = 0; n < ship.getGadgets().length; n++) {
            int[] thegadgets = ship.getGadgets();
            gadgets += "" + thegadgets[n];
            gadgets += "~";
        }
        if (data[2].equals("f")) {
            ship.setFuelCurrent(server.getGameworld().calculateFuel(ship,
                    oldPlanet, planet));
        }
        sendMessage("APPROACHDATA" + "~" + ship.getShiptype() + "~"
                + ship.getMaxCargo() + "~" + cargosize + "~" + cargo
                + "~" + ship.getShieldstrength()
                + "~" + ship.getHitpointsMax()
                + "~" + ship.getHitpointsCurrent()
                + "~" + aplayer.getReputation()
                + "~" + weapons + "~" + ship.getFuelMax()
                + "~" + ship.getFuelCurrent()
                + "~" + gadgets
                + aplayer.getPiloting()
                + "~" + aplayer.getEngineering()
                + "~" + aplayer.getTrading()
                + "~" + aplayer.getCombat());
    }

    /**
     * This gets updated information after the player goes to the
     * next zone or port.
     * @param data  UPDATEDINFO~cargo~ENDCARGO~fuel~hp~rep~currentLoc
     */
    public void getUpdatedInfo(String[] data) {
        Player aplayer = database.getPlayer(playerName);
        Ship ship = aplayer.getShip();

        int index = 1;
        boolean cargoloop = true;
        List cargo = new ArrayList();
        if (data[index].equals("ENDCARGO")) {
            index++;
        } else {
            while (cargoloop) {
                if (data[index].equals("ENDCARGO")) {
                    cargoloop = false;
                } else {
                    cargo.add(new TradeGood(data[index],
                        Integer.parseInt(data[index + 1])));
                }
                index += 2;
            }
            index--;
        }
        ship.setCargo(cargo);
        ship.setFuelCurrent(Integer.parseInt(data[index++]));
        ship.setHitpointsCurrent(Integer.parseInt(data[index++]));

        // did the player lose?
        if (Integer.parseInt(data[index - 1]) == 0) {
            int[] gadgets = ship.getGadgets();
            if (gadgets[0] == 2) {
                gadgets[0] = 0;
                // TODO go to port, lose ship, buy new
                //      ship for 2000, if can't afford
                //      you lose
                if (aplayer.spend(2000) == 0) {
                    // let the player know they lost their ship &
                    // now have a trader
                    aplayer.setShip(new Ship("Trader", aplayer));
                    ship.setGadgets(gadgets);
                    String msg = "PORTDATA~" + player.getCurrentPlanet();
                    String[] blah = msg.split("~");
                    sendPortData(blah);
                    sendMessage("X~DIED");
                } else {
                    sendMessage("X~LOST~nomoney");
                }
            } else {
                sendMessage("X~LOST~noescapepod");
            }
        }

        aplayer.setReputation(Integer.parseInt(data[index++]));
        aplayer.setCurrentLocation(data[index++]);
        if (data[index++].equals("t")) {
            Bank.loanFeeCharge(aplayer);
        }
    }

    /**
     * When a command is received to shutdown, this method closes all sockets
     * and sets run to false to end thread execution.
     */
    public void closeConnection() {
        server.disconnectUser(playerName);
        try {
            run = false;
            input.close();
            output.close();
            connection.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Sends a message to the user connected on this client socket.
     * 
     * @param messagetosend
     *            the message
     */
    public void sendMessage(String messagetosend) {
        System.out.println("Server Sent: " + messagetosend);
        output.println(messagetosend);
        output.flush();
    }

    /**
     * Returns the username of the player connected to this socket.
     * 
     * @return username the name
     */
    public String getUsername() {
        return playerName;
    }

    /**
     * @param gameName
     *            The gameName to set.
     */
    public void setGameName(String gameName) {
        this.gameName = gameName;
    }

    /**
     * @return Returns the database.
     */
    public PlayerDatabase getDatabase() {
        return database;
    }
    /**
     * @return Returns the connection.
     */
    public Socket getConnection() {
        return connection;
    }
    /**
     * @param connection The connection to set.
     */
    public void setConnection(Socket connection) {
        this.connection = connection;
    }
    /**
     * @return Returns the input.
     */
    public BufferedReader getInput() {
        return input;
    }
    /**
     * @param input The input to set.
     */
    public void setInput(BufferedReader input) {
        this.input = input;
    }
    /**
     * @return Returns the message.
     */
    public String[] getMessage() {
        return message;
    }
    /**
     * @param message The message to set.
     */
    public void setMessage(String[] message) {
        this.message = message;
    }
    /**
     * @return Returns the objectInput.
     */
    public ObjectInputStream getObjectInput() {
        return objectInput;
    }
    /**
     * @param objectInput The objectInput to set.
     */
    public void setObjectInput(ObjectInputStream objectInput) {
        this.objectInput = objectInput;
    }
    /**
     * @return Returns the objectOutput.
     */
    public ObjectOutputStream getObjectOutput() {
        return objectOutput;
    }
    /**
     * @param objectOutput The objectOutput to set.
     */
    public void setObjectOutput(ObjectOutputStream objectOutput) {
        this.objectOutput = objectOutput;
    }
    /**
     * @return Returns the output.
     */
    public PrintWriter getOutput() {
        return output;
    }
    /**
     * @param output The output to set.
     */
    public void setOutput(PrintWriter output) {
        this.output = output;
    }
    /**
     * @return Returns the player.
     */
    public Player getPlayer() {
        return player;
    }
    /**
     * @param player The player to set.
     */
    public void setPlayer(Player player) {
        this.player = player;
    }
    /**
     * @return Returns the playerName.
     */
    public String getPlayerName() {
        return playerName;
    }
    /**
     * @param playerName The playerName to set.
     */
    public void setPlayerName(String playerName) {
        this.playerName = playerName;
    }
    /**
     * @return Returns the run.
     */
    public boolean isRun() {
        return run;
    }
    /**
     * @param run The run to set.
     */
    public void setRun(boolean run) {
        this.run = run;
    }
    /**
     * @return Returns the server.
     */
    public Server getServer() {
        return server;
    }
    /**
     * @param server The server to set.
     */
    public void setServer(Server server) {
        this.server = server;
    }
    /**
     * @return Returns the gameName.
     */
    public String getGameName() {
        return gameName;
    }
    /**
     * @param database The database to set.
     */
    public void setDatabase(PlayerDatabase database) {
        this.database = database;
    }
}