package edu.cs2335.SWATLP.server.main;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import edu.cs2335.SWATLP.server.port.TradeGood;

/**
 * The ship.
 */
public class Ship implements Serializable {

    /** The type of ship. */
    private String shiptype;

    /** Maximum capacity of cargo. */
    private int maxCargo;

    /** Cargo size */
    private int cargoSize;
    
    /** The ship's cargo/inventory. */
    private List cargo;

    /** The ship's max number of gun slots. */
    private int gunslotsMax;

    /** The ship's current number of gun slots. */
    private int gunslotsCurrent;

    /** The strength of the ship's shield. */
    private int shieldstrength;

    /** The ship's maximum hit points. */
    private int hitpointsMax;

    /** The ship's current hit points. */
    private int hitpointsCurrent;

    /** The ship's different weapons. */
    private List weapons;

    /** The ship's maximum fuel level. */
    private int fuelMax;

    /** The ship's current fuel. */
    private int fuelCurrent;

    /** The gadgets attach to the ship */
    private int[] gadgets;

    /** The player. */
    private Player theplayer;

    /** The x location of the ship. */
    private int x;

    /** The y location of the ship. */
    private int y;

    /**
     * The ship's constructor.
     * 
     * @param shiptype
     *            The type of the ship.
     * @param aplayer
     *            The player owning the ship.
     */
    public Ship(String shiptype, Player aplayer) {
        this.theplayer = aplayer;
        this.shiptype = shiptype;
        weapons = new LinkedList();
        if (shiptype.equals("Trader")) {
            gunslotsMax = 3;
            maxCargo = 100;
            hitpointsMax = 80;
            addToWeapons(new Weapon("CANNON"));
            gunslotsCurrent = 1;
            fuelMax = 100;
        } else if (shiptype.equals("Fighter")) {
            gunslotsMax = 10;
            maxCargo = 50;
            hitpointsMax = 120;
            addToWeapons(new Weapon("CANNON"));
            addToWeapons(new Weapon("CANNON"));
            gunslotsCurrent = 2;
            fuelMax = 100;
        } else if (shiptype.equals("Ultimate")) {
            gunslotsMax = 20;
            maxCargo = 100;
            hitpointsMax = 200;
            addToWeapons(new Weapon("CANNON"));
            addToWeapons(new Weapon("CANNON"));
            addToWeapons(new Weapon("CANNON"));
            addToWeapons(new Weapon("CANNON"));
            addToWeapons(new Weapon("CANNON"));
            gunslotsCurrent = 5;
            fuelMax = 200;
        }
        hitpointsCurrent = hitpointsMax;
        fuelCurrent = fuelMax;
        shieldstrength = 1;

        cargo = new ArrayList();
        gadgets = new int[7];
        for (int n = 0; n < gadgets.length; n++) {
            gadgets[n] = 0;
        }
        

    }

    /**
     * Adds to the ship's weapons.
     * 
     * @param aweapon
     *            The weapon to add.
     * @return Returns a success/failure
     */
    public final int addToWeapons(Weapon aweapon) {
        if (gunslotsCurrent + aweapon.getGunslots() <= gunslotsMax) {
            weapons.add(aweapon);
            gunslotsCurrent += aweapon.getGunslots();
            return 0; // added to end
        } else {
            return -1; // full weapons
        }
    }

    /**
     * Adds a trade good to the cargo of the ship.
     * 
     * @param toadd
     *            The TradeGood to add.
     * @return Returns success/failure.
     */
    public int addToCargo(TradeGood toadd) {
        TradeGood good = null;
        //List cargo = getCargo();
        //int cargoSize = getCargoSize();
        //int maxCargo = getMaxCargo();
        
        //gets the good that we are adding to
        for (int i = 0; i < cargo.size(); i++) {
            if (((TradeGood) cargo.get(i)).getGoodtype()
                    .equals(toadd.getGoodtype())) {
                good = (TradeGood) cargo.remove(i);
                break;
            }
        }
        //found the good
        System.out.println("cargo:" + (toadd.getAmount() + cargoSize) 
                + " good:" + toadd.getGoodtype() + " max cargo:" + maxCargo);
        
        if (good == null) { //exist so add to existing
            if (toadd.getAmount() + cargoSize <= maxCargo) {
                cargo.add(toadd);
                cargoSize += toadd.getAmount();
                setCargo(cargo);
                setCargoSize(cargoSize);
            } else {
                System.out.println("full");
                return -1;
            }
        } else { //doesn't exist so add to list
            if (toadd.getAmount() + cargoSize <= maxCargo) {
                int amt = toadd.getAmount();
                toadd.setAmount(good.getAmount() + amt);
                cargo.add(toadd);
                cargoSize += amt;
                setCargo(cargo);
                setCargoSize(cargoSize);
            } else {
                cargo.add(good); //add it back
                setCargo(cargo);
                return -1;
            }
        }

        return 0;
    }

    /**
     * Removes a trade good from the cargo of the ship.
     * 
     * @param toremove
     *            The TradeGood to remove.
     * @return Returns success/failure.
     */
    public int removeCargo(TradeGood toremove) {
        for (int n = 0; n < cargo.size(); n++) {
            if (((TradeGood) cargo.get(n)).getGoodtype()
                    .equals(toremove.getGoodtype())) {
                TradeGood good = (TradeGood) cargo.remove(n);
                int amount = good.getAmount();
                if (toremove.getAmount() < amount) {
                    good.setAmount(amount - toremove.getAmount());
                    cargo.add(good);
                    cargoSize -= toremove.getAmount();
                    return 0;
                } else if (toremove.getAmount() == amount) {
                    //cargo.remove(n); already removed
                    cargoSize -= amount;
                    return 0;
                } else {
                    return -1; //not enought goods
                }
            }
        }
        return -2; //don't exist
    }

    /**
     * Upgrades the shield of the ship.
     * 
     * @param newstrength
     *            The new strength of the shield.
     */
    public void upgradeShield(int newstrength) {
        shieldstrength = newstrength;
    }

    /**
     * Refuels the ship.
     */
    public void refuel() {
        fuelCurrent = fuelMax;
    }

    /**
     * Calculates the net worth of the ship.
     * 
     * @return Returns the net worth.
     */
    public int calculateNetWorth() {
        int netWorth = 0;

        if (shiptype.equals("Trader")) {
            netWorth += 2000;
        } else if (shiptype.equals("Fighter")) {
            netWorth += 3000;
        } else if (shiptype.equals("Ultimate")) {
            netWorth += 10000;
        }
        for (int n = 0; n < cargo.size(); n++) {
            netWorth += ((TradeGood) cargo.get(n)).getValue();
        }
        for (int n = 0; n < weapons.size(); n++) {
            Weapon weapon = (Weapon) weapons.get(n);
            if (weapon.getName().equals("cannon")) {
                netWorth += 1000;
            } else if (weapon.getName().equals("ultimate")) {
                netWorth += 10000;   
            }
        }
        if (gadgets[0] == 1) {
            netWorth += 1000;
        } else if (gadgets[1] == 2) {
            netWorth += 500;
        } else if (gadgets[2] == 2) {
            netWorth += 500;
        } else if (gadgets[3] == 2) {
            netWorth += 500;
        } else if (gadgets[4] == 2) {
            netWorth += 500;
        } else if (gadgets[5] == 2) {
            netWorth += 750;
        } else if (gadgets[6] == 2) {
            netWorth += 900;
        }

        return netWorth;
    }

    /**
     * @return Returns a string containing the ship info.
     */
    public String toString() {
        String temp = "";
        temp += shiptype + "~";
        temp += "" + shieldstrength + "~";
        temp += "" + gunslotsCurrent + "~" + maxCargo + "~";
        temp += "" + fuelCurrent + "~" + fuelMax + "~";
        temp += "" + hitpointsCurrent + "~" + hitpointsMax + "~";
        for (int i = 0; i < weapons.size(); i++) {
            temp += ((Weapon) weapons.get(i)).getName() + "~";
        }

        return temp;
    }

    /**
     * @return Returns the type of ship.
     */
    public String getShiptype() {
        return shiptype;
    }

    /**
     * @return Returns the current size of the cargo space.
     */
    public int getCargoSize() {
        return cargoSize;
    }

    /**
     * @return Returns the cargo of the ship.
     */
    public List getCargo() {
        return cargo;
    }

    /**
     * @param cargo
     *            Sets the cargo of the ship.
     */
    public void setCargo(List cargo) {
        this.cargo = cargo;
    }

    /**
     * Setter for the hitpoints of the ship
     * 
     * @param amount
     *            The amount to set the hitpoints to.
     */
    public void setHitpointsCurrent(int amount) {
        hitpointsCurrent = amount;
    }

    /**
     * @return Returns the max hit points of the ship.
     */
    public int getHitpointsMax() {
        return hitpointsMax;
    }

    /**
     * @return Returns the current hit points of the ship.
     */
    public int getHitpointsCurrent() {
        return hitpointsCurrent;
    }

    /**
     * @return Returns the gadgets.
     */
    public int[] getGadgets() {
        return gadgets;
    }

    /**
     * @param  gadget  index
     */
    public void addGadgetPermanently(int gadget) {
        gadgets[gadget] = 2;
    }
    /**
     * @param thegadgets
     *            The gadgets to set.
     */
    public void setGadgets(int[] thegadgets) {
        this.gadgets = thegadgets;

        // Navigation System
        if (gadgets[0] == 1) {
            addGadgetPermanently(0);
        }
        if (gadgets[1] == 1) {
            theplayer.setPiloting(theplayer.getPiloting() + 2);
            addGadgetPermanently(1);
        }
        // Mechanics Kit
        if (gadgets[2] == 1) {
            theplayer.setEngineering(theplayer.getEngineering() + 2);
            addGadgetPermanently(2);
        }
        // Negotiations
        if (gadgets[3] == 1) {
            theplayer.setTrading(theplayer.getTrading() + 2);
            addGadgetPermanently(3);
        }
        // Targeting System
        if (gadgets[4] == 1) {
            theplayer.setCombat(theplayer.getCombat() + 2);
            addGadgetPermanently(4);
        }
        // Cargo Compressor
        if (gadgets[5] == 1) {
            maxCargo += 50;
            addGadgetPermanently(5);
        }
        // External Turrets
        if (gadgets[6] == 1) {
            gunslotsMax += 5;
            addGadgetPermanently(6);
        }
    }
    /**
     * @param thegadgets  the gadgets to transfer
     */
    public void transferGadgets(int[] thegadgets) {
        if (thegadgets[0] == 2) {
            this.gadgets[0] = 1;
        } else if (thegadgets[1] == 2) {
            this.gadgets[1] = 2;
        } else if (thegadgets[2] == 2) {
            this.gadgets[2] = 2;
        } else if (thegadgets[3] == 2) {
            this.gadgets[3] = 2;
        } else if (thegadgets[4] == 2) {
            this.gadgets[4] = 2;
        } else if (thegadgets[5] == 2) {
            this.gadgets[5] = 1;
        } else if (thegadgets[6] == 2) {
            this.gadgets[6] = 1;
        }
        setGadgets(this.gadgets);
    }

    /**
     * @return Returns the weapons.
     */
    public List getWeapons() {
        return weapons;
    }

    /**
     * @param weapons
     *            The weapons to set.
     */
    public void setWeapons(List weapons) {
        this.weapons = weapons;
    }

    /**
     * @return Returns the damage done to the ship.
     */
    public int getDamage() {
        return hitpointsMax - hitpointsCurrent;
    }

    /**
     * @return Returns the shieldstrength.
     */
    public int getShieldstrength() {
        return shieldstrength;
    }

    /**
     * @return Returns the fuelCurrent.
     */
    public int getFuelCurrent() {
        return fuelCurrent;
    }

    /**
     * @param fuelCurrent
     *            The fuelCurrent to set.
     */
    public void setFuelCurrent(int fuelCurrent) {
        this.fuelCurrent = fuelCurrent;
    }

    /**
     * @return Returns the x.
     */
    public int getX() {
        return x;
    }

    /**
     * @param x
     *            The x to set.
     */
    public void setX(int x) {
        this.x = x;
    }

    /**
     * @return Returns the y.
     */
    public int getY() {
        return y;
    }

    /**
     * @param y
     *            The y to set.
     */
    public void setY(int y) {
        this.y = y;
    }
    /**
     * @return Returns the fuelMax.
     */
    public int getFuelMax() {
        return fuelMax;
    }
    /**
     * @param fuelMax The fuelMax to set.
     */
    public void setFuelMax(int fuelMax) {
        this.fuelMax = fuelMax;
    }
    /**
     * @return Returns the theplayer.
     */
    public Player getTheplayer() {
        return theplayer;
    }
    /**
     * @return Returns the maxCargo.
     */
    public int getMaxCargo() {
        return maxCargo;
    }
    /**
     * @param maxCargo The maxCargo to set.
     */
    public void setMaxCargo(int maxCargo) {
        this.maxCargo = maxCargo;
    }
    /**
     * @param cargoSize  cargo size to set
     */
    public void setCargoSize(int cargoSize) {
        this.cargoSize = cargoSize;
    }
    /**
     * @return Returns the gunslotsCurrent.
     */
    public int getGunslotsCurrent() {
        return gunslotsCurrent;
    }
    /**
     * @param gunslotsCurrent The gunslotsCurrent to set.
     */
    public void setGunslotsCurrent(int gunslotsCurrent) {
        this.gunslotsCurrent = gunslotsCurrent;
    }
    /**
     * @return Returns the gunslotsMax.
     */
    public int getGunslotsMax() {
        return gunslotsMax;
    }
    /**
     * @param gunslotsMax The gunslotsMax to set.
     */
    public void setGunslotsMax(int gunslotsMax) {
        this.gunslotsMax = gunslotsMax;
    }
    /**
     * @param hitpointsMax The hitpointsMax to set.
     */
    public void setHitpointsMax(int hitpointsMax) {
        this.hitpointsMax = hitpointsMax;
    }
    /**
     * @param shieldstrength The shieldstrength to set.
     */
    public void setShieldstrength(int shieldstrength) {
        this.shieldstrength = shieldstrength;
    }
    /**
     * @param shiptype The shiptype to set.
     */
    public void setShiptype(String shiptype) {
        this.shiptype = shiptype;
    }
    /**
     * @param theplayer The theplayer to set.
     */
    public void setTheplayer(Player theplayer) {
        this.theplayer = theplayer;
    }
}