package edu.cs2335.SWATLP.server.main;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

/**
 * The universe, man!
 */
public class Universe implements Serializable {
    /** The size of the universe */
    public static final int MAX_SIZE = 50;

    /** The planets in the universe. */
    private List planets;

    /** The planets in the universe. */
    private Hashtable planetss;

    /** The double array of planets locations */
    private String[][] planetsLoc;

    /** The list of all the planets name */
    private List planetsName;

    /**
     * Universe's Constructor
     */
    public Universe() {
        planetsLoc = new String[MAX_SIZE][MAX_SIZE];
        planetss = new Hashtable();
        planets = new ArrayList();
        planetsName = new ArrayList();
        generatePlanets();
    }

    /**
     * Get the planets within the distance given.
     * 
     * @param aship
     *            The ship getting the range.
     * @return Returns a list of planets within the range.
     */
    public List getPlanetsInRange(Ship aship) {
        List planetsinrange = new ArrayList();

        for (int n = 0; n < planets.size(); n++) {
            Planet aplanet = (Planet) planets.get(n);
            int xdist = (aship.getX() - aplanet.getXlocation());
            int ydist = (aship.getY() - aplanet.getYlocation());
            int dist = (int) Math.sqrt((xdist * xdist) + (ydist * ydist));
            int fuel = aship.getFuelCurrent();
            System.out.println("fuel remaining:" + fuel);
            if (fuel - dist >= 0) {
                planetsinrange.add(aplanet);
            }
        }

        return planetsinrange;
    }

    /**
     * Fuel calculation
     * @param aship    ship travelling
     * @param aplanet  planet
     * @return Returns the new fuel.
     * @param oldone o
     */
    public int calculateFuel(Ship aship, Planet oldone, Planet aplanet) {
        int xdist = (oldone.getXlocation() - aplanet.getXlocation());
        int ydist = (oldone.getYlocation() - aplanet.getYlocation());
        int dist = (int) Math.sqrt((xdist * xdist) + (ydist * ydist));
        int fuel = aship.getFuelCurrent();
        if (fuel - dist >= 0) {
            if (aship.getGadgets()[1] == 1) {
                return fuel - dist + 10;
            }
            return fuel - dist;
        } else {
            return -1;
        }
    }

    /**
     * Generates the planets for the universe.
     */
    private final void generatePlanets() {
        for (int i = 0; i < MAX_SIZE; i++) {
            for (int j = 0; j < MAX_SIZE; j++) {
                planetsLoc[i][j] = "o";
            }
        }

        addPlanet("Zebes", "Capitalism", "High Tech");
        addPlanet("Mars", "Pacifism", "Low Tech");
        addPlanet("LSD", "Communism", "Low Tech");
        addPlanet("Beatles", "Pacifism", "Medieval");
        addPlanet("GATech", "Cybernetic", "High Tech");
        addPlanet("X", "Anarchy", "Medieval");
        addPlanet("Pontais", "Pacifism", "Medieval");
        addPlanet("Katina", "Capitalism", "Low Tech");
        addPlanet("Venom", "Anarchy", "High Tech");
        addPlanet("UGA", "Anarchy", "Medieval");
        
    }

    /**
     * Adds a new planet to the universe.
     * 
     * @param name
     *            The name of the planet.
     * @param gov
     *            The government of the planet.
     * @param tech
     *            The tech level of the planet.
     * @return Returns if the planet was created or not.
     */
    public final boolean addPlanet(String name, 
            String gov, String tech) {
        if (planetss.get(name) != null) {
            return false;
        }

        boolean placed = false;

        // default location
        int x = 0;
        int y = 0;

        // this looks at a place in the universe and sees if a planets there
        while (!placed) {
            x = (int) (Math.random() * MAX_SIZE);
            y = (int) (Math.random() * MAX_SIZE);
            if (planetsLoc[x][y].equals("o")) { //found a empty area in space
                planetsLoc[x][y] = "x";
                break;
            }
        }

        Planet toadd = new Planet(name, x, y, gov, tech);

        planetsName.add(name); //List of planets name
        planetss.put(name, toadd); //HashTable of planets data
        planets.add(toadd);
        return true;
    }

    /**
     * @param planetname
     *            name
     * @param zonenumber
     *            num
     * @return activate
     */
    /*
     * public Planet activateZone(String planetname, int zonenumber) { Planet
     * toactivate = getPlanet(planetname); Zone newzone = new Zone();
     * toactivate.setZoneActive(newzone, zonenumber); return toactivate; }
     */

    /**
     * @param planetname
     *            name
     * @param zonenumber
     *            num
     */
    /*
     * public void deactivateZone(String planetname, int zonenumber) { Planet
     * todeactivate = getPlanet(planetname);
     * todeactivate.setZoneInactive(zonenumber); }
     */
    /**
     * @param planetname
     *            name
     * @param zonenumber
     *            num
     * @return active
     */
    /*
     * public boolean checkZone(String planetname, int zonenumber) { Planet
     * tocheck = getPlanet(planetname); return tocheck.isActive(zonenumber); }
     */
    /**
     * Gets the home planet
     * 
     * @param planetName
     *            the name of the planet
     * @return Returns a specific planet
     */
    public Planet getPlanet(String planetName) {
        //return (Planet) planetss.get(planetName);

        System.out.println("planet size = " + planets.size());
        for (int n = 0; n < planets.size(); n++) {
            if (((Planet) planets.get(n)).getName().equals(planetName)) {
                return (Planet) planets.get(n);
            }
        }

        return null;
    }
    /**
     * @return Returns the planets.
     */
    public List getPlanets() {
        return planets;
    }
    /**
     * @param planets The planets to set.
     */
    public void setPlanets(List planets) {
        this.planets = planets;
    }
    /**
     * @return Returns the planetsLoc.
     */
    public String[][] getPlanetsLoc() {
        return planetsLoc;
    }
    /**
     * @param planetsLoc The planetsLoc to set.
     */
    public void setPlanetsLoc(String[][] planetsLoc) {
        this.planetsLoc = planetsLoc;
    }
    /**
     * @return Returns the planetsName.
     */
    public List getPlanetsName() {
        return planetsName;
    }
    /**
     * @param planetsName The planetsName to set.
     */
    public void setPlanetsName(List planetsName) {
        this.planetsName = planetsName;
    }
    /**
     * @return Returns the planetss.
     */
    public Hashtable getPlanetss() {
        return planetss;
    }
    /**
     * @param planetss The planetss to set.
     */
    public void setPlanetss(Hashtable planetss) {
        this.planetss = planetss;
    }
}