package edu.cs2335.SWATLP.server.port;

import java.util.List;

import edu.cs2335.SWATLP.server.main.Player;
import edu.cs2335.SWATLP.server.main.Ship;

/**
 * The Jail!!
 */
public class Jail {

    /**
     * Jail Constructor
     */
    private Jail() {
    }

    /**
     * Locks the player in jail.
     * 
     * @param criminal
     *            The player being locked up.
     */
    public static void lockup(Player criminal) {
        criminal.depositMoneyIntoBank(criminal.getCashonhand());
        criminal.setFine(criminal.getReputation() * 1000);
        criminal.setInjail(true);
    }

    /**
     * Releases the player from jail.
     * 
     * @param criminal
     *            The player being released.
     */
    public static void release(Player criminal) {
        criminal.setFine(0);
        criminal.setInjail(false);
    }

    /**
     * Pay fine from bank account funds.
     * 
     * @param criminal
     *            The player paying the fines.
     * @return returns released/nodeal
     */
    public static int payFine(Player criminal) {
        if (criminal.getBankaccount() >= criminal.getFine()) {
            criminal.setBankaccount(criminal.getBankaccount()
                    - criminal.getFine());
            release(criminal);
            return 0;
        } else {
            return -1;
        }
    }

    /**
     * Sell ship/cargo
     * 
     * @param criminal
     *            The criminal selling his stuff
     * @return Returns released/nodeal
     */
    public static int sellStuff(Player criminal) {
      //  int total = 0;

        // check the cargo, deposit money into account on each item sold
        List cargo = criminal.getShip().getCargo();
        for (int n = 0; n < cargo.size(); n++) {
            int tradegoodvalue = ((TradeGood) cargo.get(n)).getValue();
            criminal.depositMoneyIntoBank(tradegoodvalue);
            cargo.remove(n);
            if (criminal.getBankaccount() >= criminal.getFine()) {
                criminal.setBankaccount(criminal.getBankaccount()
                        - criminal.getFine());
                release(criminal);
                return 0;
            }
        }

        // check the ship, deposit money into bank account
        int shipvalue = criminal.getShip().calculateNetWorth();
        criminal.setShip(null);
        criminal.depositMoneyIntoBank(shipvalue);
        if (criminal.getBankaccount() >= criminal.getFine()) {
            criminal.setBankaccount(criminal.getBankaccount()
                    - criminal.getFine());
            release(criminal);
            return 0;
        } else {
            return -1;
        }
    }

    /**
     * Declare bankruptcy
     * 
     * @param criminal
     *            The player declaring
     */
    public static void declareBankruptcy(Player criminal) {
        criminal.setBankaccount(0);
        criminal.setCashonhand(0);
        criminal.setShip(new Ship("Trader", criminal));
        criminal.setInjail(false);
        criminal.setCurrentLocation(criminal.getCurrentLocation() + "~port");
    }
}