package edu.cs2335.SWATLP.server.port;

import java.util.ArrayList;
import java.util.List;

import edu.cs2335.SWATLP.server.main.Player;
import edu.cs2335.SWATLP.server.main.Ship;
import edu.cs2335.SWATLP.server.main.Weapon;

/**
 * The shipyard
 */
public class Shipyard {

    /** The ships in the shipyard. */
    private static List ships;

    /**
     * Default constructor
     */
    private Shipyard() {
    }

    /**
     * Buys a new ship for the player.
     * 
     * @param buyer
     *            The buyer
     * @param shipname
     *            The shiptype
     * @param transferitems
     *            The option to transfer items for a fee.
     * @return Returns a success/failure code.
     */
    public static int buyShip(Player buyer, String shipname,
            boolean transferitems) {
        List cargo = buyer.getShip().getCargo();
        List weapons = buyer.getShip().getWeapons();
        int[] gadgets = buyer.getShip().getGadgets();

        if (shipname.equals("Trader")) {
            int spending = 1;
            if (buyer.getShip().getShiptype().equals("Trader")) {
                if (transferitems) {
                    spending = buyer.spend(500);
                } else {
                    spending = 0;
                }
            } else if (buyer.getShip().getShiptype().equals("Fighter")) {
                if (transferitems) {
                    buyer.addMoney(500);
                } else {
                    buyer.addMoney(1000);
                }
                spending = 0;
            } else if (buyer.getShip().getShiptype().equals("Ultimate")) {
                if (transferitems) {
                    buyer.addMoney(7500);
                } else {
                    buyer.addMoney(8000);
                }
                spending = 0;
            }
            if (spending == 0) {
                buyer.setShip(new Ship("Trader", buyer));
                buyer.getShip().setCargo(cargo);
                if (transferitems) {
                    buyer.getShip().setWeapons(weapons);
                    buyer.getShip().transferGadgets(gadgets);
                }
                return 0;
            } else {
                return -1;
            }
        } else if (shipname.equals("Fighter")) {
            int spending = 1;
            if (buyer.getShip().getShiptype().equals("Trader")) {
                if (transferitems) {
                    spending = buyer.spend(1500);
                } else {
                    spending = buyer.spend(1000);
                }
            } else if (buyer.getShip().getShiptype().equals("Fighter")) {
                if (transferitems) {
                    spending = buyer.spend(500);
                } else {
                    spending = 0;
                }
            } else if (buyer.getShip().getShiptype().equals("Ultimate")) {
                if (transferitems) {
                    buyer.addMoney(6500);
                } else {
                    buyer.addMoney(7000);
                }
                spending = 0;
            }
            if (spending == 0) {
                buyer.setShip(new Ship("Fighter", buyer));
                buyer.getShip().setCargo(cargo);
                if (transferitems) {
                    buyer.getShip().setWeapons(weapons);
                    buyer.getShip().transferGadgets(gadgets);
                }
                return 0;
            } else {
                return -1;
            }
        } else if (shipname.equals("Ultimate")) {
            int spending = 1;
            if (buyer.getShip().getShiptype().equals("Trader")) {
                if (transferitems) {
                    spending = buyer.spend(8500);
                } else {
                    spending = buyer.spend(8000);
                }
            } else if (buyer.getShip().getShiptype().equals("Fighter")) {
                if (transferitems) {
                    spending = buyer.spend(7500);
                } else {
                    spending = buyer.spend(7000);
                }
            } else if (buyer.getShip().getShiptype().equals("Ultimate")) {
                if (transferitems) {
                    spending = buyer.spend(500);
                } else {
                    spending = 0;
                }
            }
            if (spending == 0) {
                buyer.setShip(new Ship("Ultimate", buyer));
                buyer.getShip().setCargo(cargo);
                if (transferitems) {
                    buyer.getShip().setWeapons(weapons);
                    buyer.getShip().transferGadgets(gadgets);
                }
                return 0;
            } else {
                return -1;
            }
        } else {
            return -2;
        }
    }

    /**
     * Buys a new gadget.
     * 
     * @param buyer
     *            The buyer
     * @param gadgetname
     *            The gadget
     * @return Returns success/failure.
     */
    public static int buyGadget(Player buyer, String gadgetname) {
        int[] gadgets = buyer.getShip().getGadgets();
        if (gadgetname.equals("escapepod")) {
            if (gadgets[0] == 2) {
                return -1;
            }
            if (buyer.spend(1000) == 0) {
                gadgets[0] = 1;
                buyer.getShip().setGadgets(gadgets);
                return 0;
            } else {
                return -1;
            }
        } else if (gadgetname.equals("navigationalsystem")) {
            if (gadgets[1] == 2) {
                return -1;
            }
            if (buyer.spend(500) == 0) {
                gadgets[1] = 1;
                buyer.getShip().setGadgets(gadgets);
                return 0;
            } else {
                return -1;
            }
        } else if (gadgetname.equals("mechanicskit")) {
            if (gadgets[2] == 2) {
                return -1;
            }
            if (buyer.spend(500) == 0) {
                gadgets[2] = 1;
                buyer.getShip().setGadgets(gadgets);
                return 0;
            } else {
                return -1;
            }
        } else if (gadgetname.equals("negotiationschip")) {
            if (gadgets[3] == 2) {
                return -1;
            }
            if (buyer.spend(500) == 0) {
                gadgets[3] = 1;
                buyer.getShip().setGadgets(gadgets);
                return 0;
            } else {
                return -1;
            }
        } else if (gadgetname.equals("targetingsystem")) {
            if (gadgets[4] == 2) {
                return -1;
            }
            if (buyer.spend(500) == 0) {
                gadgets[4] = 1;
                buyer.getShip().setGadgets(gadgets);
                return 0;
            } else {
                return -1;
            }
        } else if (gadgetname.equals("cargocompressor")) {
            if (gadgets[5] == 2) {
                return -1;
            }
            if (buyer.spend(750) == 0) {
                gadgets[5] = 1;
                buyer.getShip().setGadgets(gadgets);
                return 0;
            } else {
                return -1;
            }
        } else if (gadgetname.equals("externalturrets")) {
            if (gadgets[6] == 2) {
                return -1;
            }
            if (buyer.spend(900) == 0) {
                gadgets[6] = 1;
                buyer.getShip().setGadgets(gadgets);
                return 0;
            } else {
                return -1;
            }
        } else {
            return -2;
        }
    }

    /**
     * Buys a new weapon.
     * 
     * @param buyer
     *            The buyer
     * @param weaponname
     *            Name of the weapon
     * @return Returns a success/failure code.
     */
    public static int buyWeapon(Player buyer, String weaponname) {
        if (weaponname.equals("Ultimate")) {
            if (buyer.spend(10000) == 0) {
                buyer.getShip().addToWeapons(new Weapon("Ultimate"));
                return 0;
            } else {
                return -1;
            }
        } else if (weaponname.equals("Cannon")) {
            if (buyer.spend(1000) == 0) {
                buyer.getShip().addToWeapons(new Weapon("Cannon"));
                return 0;
            } else {
                return -1;
            }
        } else {
            return -2;
        }
    }

    /**
     * Refuels the ship.
     * @param refueler  refuel dude
     * @return Returns the cost/failure
     */
    public static int refuelShip(Player refueler) {
        int fuel = refueler.getShip().getFuelCurrent();
        int cost = fuel * 2;
        if (refueler.spend(cost) == 0) {
            refueler.getShip().setFuelCurrent(
                    refueler.getShip().getFuelMax());
            return cost;
        } else {
            return -1;
        }
    }

    /**
     * Repairs the ship.
     * 
     * @param repairer
     *            The player repairing.
     * @return Returns the cost/failure
     */
    public static int repairShip(Player repairer) {
        int dmg = repairer.getShip().getDamage();
        int cost = dmg * 4;
        if (repairer.spend(cost) == 0) {
            repairer.getShip().setHitpointsCurrent(
                    repairer.getShip().getHitpointsMax());
            return cost;
        } else {
            return -1;
        }
    }

    /**
     * @param theplayer  player
     * @return success/failure
     */
    public static int upgradeShields(Player theplayer) {
        int ss = theplayer.getShip().getShieldstrength();
        if (ss == 1) {
            if (theplayer.spend(2000) == 0) {
                theplayer.getShip().setShieldstrength(ss + 1);
                return 2000;
            } else {
                return -1;
            }
        } else if (ss == 2) {
            if (theplayer.spend(5000) == 0) {
                theplayer.getShip().setShieldstrength(ss + 1);
                return 5000;
            } else {
                return -1;
            }
        } else if (ss == 3) {
            if (theplayer.spend(10000) == 0) {
                theplayer.getShip().setShieldstrength(ss + 1);
                return 10000;
            } else {
                return -1;
            }
        } else if (ss == 4) {
            if (theplayer.spend(15000) == 0) {
                theplayer.getShip().setShieldstrength(ss + 1);
                return 15000;
            } else {
                return -1;
            }
        } else {
            return -2;
        }
    }

    /**
     * Removes a weapon from the ship.
     * 
     * @param theplayer
     *            The player
     * @param weapon
     *            The weapon being removed
     * @return Success/failure
     */
    public static int removeWeapon(Player theplayer, String weapon) {
        List weapons = theplayer.getShip().getWeapons();
        for (int n = 0; n < weapons.size(); n++) {
            if (((Weapon) weapons.get(n)).getName().equals(weapon)) {
                weapons.remove(n);
                return 0;
            }
        }
        return -1;
    }

    /**
     * Moves a weapon at the end of the queue to the front.
     * 
     * @param reorderer
     *            The player
     * @param index
     *            The index of the weapon.
     */
    public static void reorderWeapon(Player reorderer, int index) {
        List weapons = reorderer.getShip().getWeapons();
        Weapon old = (Weapon) weapons.remove(index);
        weapons.add(0, old);
    }

    /**
     * Modifier - adds a ship to the shipyard.
     * 
     * @param aship
     *            The ship being added.
     */
    public static void addShip(Ship aship) {
        if (ships == null) {
            ships = new ArrayList();
        }
        ships.add(aship);
    }

    /**
     * Modifier - removes a ship from the shipyard.
     * 
     * @param aship
     *            The ship being removed.
     */
    public static void removeShip(Ship aship) {
        ships.remove(aship);
    }

    /**
     * @return Returns the ships in the shipyard.
     */
    public static List getShips() {
        return ships;
    }
}

