package edu.cs2335.SWATLP.server.port;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * The warehouse
 */
public class Warehouse implements Serializable {

    /** The amount of items in the warehouse. */
    private int sizeCurrent;

    /** The actual trade goods in the warehouse. */
    private List tradegoods;

    /**
     * Constructor for the Warehouse
     */
    public Warehouse() {
        tradegoods = new ArrayList();
        sizeCurrent = 0;
    }

    /**
     * Adds an item to the warehouse.
     * 
     * @param item
     *            The item being added.
     */
    public void addItem(TradeGood item) {
        if (item.getAmount() + sizeCurrent < 1000) {
            tradegoods.add(item);
            sizeCurrent += item.getAmount();
        } else {
            // TODO display to the user that the warehouse is full
            return;
        }
    }

    /**
     * Removes an item from the warehouse.
     * 
     * @param item
     *            The trade good being removed.
     * @return Returns success/failure
     */
    public int removeItem(TradeGood item) {
        if (sizeCurrent > 0) {
            for (int n = 0; n < tradegoods.size(); n++) {
                TradeGood somegood = (TradeGood) tradegoods.get(n);
                if (somegood.getGoodtype().equals(item.getGoodtype())) {
                    if (somegood.getAmount() < item.getAmount()) {
                        // TODO Cannot remove more than you have in the
                        //       warehouse.
                        return -1;
                    } else if (somegood.getAmount() == item.getAmount()) {
                        tradegoods.remove(item);
                        sizeCurrent -= item.getAmount();
                        return 0;
                    } else {
                        somegood.setAmount(somegood.getAmount()
                                - item.getAmount());
                        sizeCurrent -= item.getAmount();
                        return 0;
                    }
                }
            }
            return -1;
        } else {
            return -1;
        }
    }

    /**
     * Displays the contents of the warehouse.
     * 
     * @return Returns a string containing the contents.
     */
    public String displayContents() {
        String goods = "" + tradegoods.size() + "~";
        for (int i = 0; i < tradegoods.size(); i++) {
            TradeGood item = (TradeGood) tradegoods.get(i);
            goods += item.getGoodtype() + "~" + item.getAmount() + "~";
        }
        return goods;
    }

    /**
     * Calculates the net worth of all the trade goods in the warehouse.
     * 
     * @return Returns the net worth.
     */
    public int calculateNetWorth() {
        int netWorth = 0;

        for (int n = 0; n < tradegoods.size(); n++) {
            netWorth += ((TradeGood) tradegoods.get(n)).getValue();
        }

        return netWorth;
    }

    /**
     * @return Returns the sizeCurrent.
     */
    public int getSizeCurrent() {
        return sizeCurrent;
    }

    /**
     * @return Returns the tradegoods.
     */
    public List getTradegoods() {
        return tradegoods;
    }
    
    /**
     * Enter an whole new list
     * @param tradegoods the new list
     */
    public void setTradegoods(List tradegoods) {
        this.tradegoods = tradegoods;
    }
    /**
     * @param sizeCurrent The sizeCurrent to set.
     */
    public void setSizeCurrent(int sizeCurrent) {
        this.sizeCurrent = sizeCurrent;
    }
}